module ExampleStuff exposing (..)

import Xml exposing (Value(..))
import Xml.Encode exposing (null)
import Xml.Decode exposing (..)
import Xml.Query exposing (..)
import Date exposing (Date, Month(Jan))
import Date.Extra as Date


rebucket : a -> a
rebucket =
    identity


type Unit
    = Bytes Int


type alias Object =
    { time : Date
    , size : Unit
    , path : List String
    }


fromXML : String -> Result String (List Object)
fromXML body =
    let
        getObject : Value -> Result String Object
        getObject value =
            let
                time =
                    tag "LastModified" (string >> (Result.andThen Date.fromString))
                        >> default (Date.fromParts 1970 Jan 1 0 0 0 0)

                size =
                    tag "Size" (int >> Result.map (Bytes >> rebucket))
                        >> default (Bytes 0)

                path =
                    tag "Key" (string >> Result.map (String.split "/"))
                        >> default []
            in
                Result.map3
                    (\time size path ->
                        { time = time
                        , size = size
                        , path = path
                        }
                    )
                    (time value)
                    (size value)
                    (path value)
    in
        decode body
            |> Result.map
                (tags "ListBucketResult"
                    >> List.map (tags "Contents")
                    >> List.concat
                    >> collect getObject
                )


stuff =
    """
<?xml version="1.0" encoding="UTF-8"?>
<ListBucketResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/"><Name>converge-builds-dl</Name><Prefix></Prefix><Marker></Marker><MaxKeys>1000</MaxKeys><IsTruncated>true</IsTruncated><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_darwin_386/converge</Key><LastModified>2016-10-27T16:09:19.000Z</LastModified><ETag>&quot;2f502a2524bef9b06cbb8aa8b820ad18&quot;</ETag><Size>10492296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_darwin_386/converge.sha256sum</Key><LastModified>2016-10-27T16:09:20.000Z</LastModified><ETag>&quot;32910ed0031ec83fbf0550b6c48f717a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_darwin_amd64/converge</Key><LastModified>2016-10-27T16:09:20.000Z</LastModified><ETag>&quot;07aa99ead2dcec8c9e123ed72dce9a67&quot;</ETag><Size>11777616</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_darwin_amd64/converge.sha256sum</Key><LastModified>2016-10-27T16:09:20.000Z</LastModified><ETag>&quot;d18a2d94741e3292ea1b7f2127b55346&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_freebsd_386/converge</Key><LastModified>2016-10-27T16:09:20.000Z</LastModified><ETag>&quot;c63cf7e6561955c66134a20bdaa8ddde&quot;</ETag><Size>9352416</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_freebsd_386/converge.sha256sum</Key><LastModified>2016-10-27T16:09:21.000Z</LastModified><ETag>&quot;6c96e5596ccdfe6df79a9688e0df08d7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_freebsd_amd64/converge</Key><LastModified>2016-10-27T16:09:21.000Z</LastModified><ETag>&quot;d6d3f55ce301d779825b57aed3bc41b0&quot;</ETag><Size>10566080</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-10-27T16:09:22.000Z</LastModified><ETag>&quot;f1c102a8b04a224e3467bcbf0914781d&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_freebsd_arm/converge</Key><LastModified>2016-10-27T16:09:22.000Z</LastModified><ETag>&quot;a9a3d26690977131435e4f8c2c2d45aa&quot;</ETag><Size>9601440</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_freebsd_arm/converge.sha256sum</Key><LastModified>2016-10-27T16:09:22.000Z</LastModified><ETag>&quot;b716b71839c873ea5e71a6829990f2fe&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_386/converge</Key><LastModified>2016-10-27T16:09:22.000Z</LastModified><ETag>&quot;deb27084c77b91658e9b85d1394f5796&quot;</ETag><Size>9426368</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_386/converge.sha256sum</Key><LastModified>2016-10-27T16:09:23.000Z</LastModified><ETag>&quot;0fa45e200bd8c71e3a2121dd30f6b829&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_amd64/converge</Key><LastModified>2016-10-27T16:09:23.000Z</LastModified><ETag>&quot;0c0152f0729aeecb1faa008240bff33d&quot;</ETag><Size>10633440</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_amd64/converge.sha256sum</Key><LastModified>2016-10-27T16:09:24.000Z</LastModified><ETag>&quot;8df41af2d6d0747eb0b8da0a5310fc47&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_arm/converge</Key><LastModified>2016-10-27T16:09:24.000Z</LastModified><ETag>&quot;39d510baa0b5f856dd77140e5ae1ccfc&quot;</ETag><Size>9597736</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_arm/converge.sha256sum</Key><LastModified>2016-10-27T16:09:24.000Z</LastModified><ETag>&quot;726efa5af99fb0473983814e63380723&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_arm64/converge</Key><LastModified>2016-10-27T16:09:24.000Z</LastModified><ETag>&quot;f2e0cfb544a66cd8f48ae7b3c074cc45&quot;</ETag><Size>10827744</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_arm64/converge.sha256sum</Key><LastModified>2016-10-27T16:09:24.000Z</LastModified><ETag>&quot;0e0aff1f31110860184a5bcf5a8a0a9e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_ppc64/converge</Key><LastModified>2016-10-27T16:09:25.000Z</LastModified><ETag>&quot;31aea8702f6f47fa5cb268a2a9fd4dab&quot;</ETag><Size>11155488</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_ppc64/converge.sha256sum</Key><LastModified>2016-10-27T16:09:25.000Z</LastModified><ETag>&quot;077c1f3ba0f5c6f4ea8e0b1dfca8ca61&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_ppc64le/converge</Key><LastModified>2016-10-27T16:09:25.000Z</LastModified><ETag>&quot;c6b1987199e5f62c7d814f5d2e52ba73&quot;</ETag><Size>11155488</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-10-27T16:09:26.000Z</LastModified><ETag>&quot;5474f59ae9a685e6f3be50a9e9500411&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_solaris_amd64/converge</Key><LastModified>2016-10-27T16:09:26.000Z</LastModified><ETag>&quot;01cdddaad7cbeb749301d286d4e9f7f3&quot;</ETag><Size>10499648</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/converge_0.3.0-1-g9379a1d_solaris_amd64/converge.sha256sum</Key><LastModified>2016-10-27T16:09:26.000Z</LastModified><ETag>&quot;319a4dc07c7bbb25c61f0d621628498a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_darwin_386.tar.gz</Key><LastModified>2016-10-27T16:09:26.000Z</LastModified><ETag>&quot;a38cfe66fbdd8603dd872feb3eafe7be&quot;</ETag><Size>3716435</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_darwin_amd64.tar.gz</Key><LastModified>2016-10-27T16:09:26.000Z</LastModified><ETag>&quot;76d52e519f9b2c386aad5181717a64c7&quot;</ETag><Size>3898734</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_freebsd_386.tar.gz</Key><LastModified>2016-10-27T16:09:27.000Z</LastModified><ETag>&quot;be20988fe985eaea650370d67b238616&quot;</ETag><Size>3523439</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_freebsd_amd64.tar.gz</Key><LastModified>2016-10-27T16:09:27.000Z</LastModified><ETag>&quot;dacbfc2e7f4d83cfda2f081d91535ade&quot;</ETag><Size>3705200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_freebsd_arm.tar.gz</Key><LastModified>2016-10-27T16:09:28.000Z</LastModified><ETag>&quot;76c34b577d1a8d50947288e345094ce7&quot;</ETag><Size>3426884</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_linux_386.tar.gz</Key><LastModified>2016-10-27T16:09:28.000Z</LastModified><ETag>&quot;938558b74ad4b9f866daed89d5614882&quot;</ETag><Size>3528808</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_linux_amd64.tar.gz</Key><LastModified>2016-10-27T16:09:28.000Z</LastModified><ETag>&quot;cb5bbfb6dbf5abb428e946d17c63ceb0&quot;</ETag><Size>3734581</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_linux_arm.tar.gz</Key><LastModified>2016-10-27T16:09:28.000Z</LastModified><ETag>&quot;c7756f570cc11cf5c7041308620c9549&quot;</ETag><Size>3430170</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_linux_arm64.tar.gz</Key><LastModified>2016-10-27T16:09:28.000Z</LastModified><ETag>&quot;07f68d6811f72180169876c08f557ef8&quot;</ETag><Size>3453644</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_linux_ppc64.tar.gz</Key><LastModified>2016-10-27T16:09:29.000Z</LastModified><ETag>&quot;98ad62e802385a80789a0aae77c5d379&quot;</ETag><Size>3470101</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_linux_ppc64le.tar.gz</Key><LastModified>2016-10-27T16:09:30.000Z</LastModified><ETag>&quot;3497c881a376d7d18cd96afc7520a805&quot;</ETag><Size>3428050</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/converge_0.3.0-1-g9379a1d_solaris_amd64.tar.gz</Key><LastModified>2016-10-27T16:09:30.000Z</LastModified><ETag>&quot;0f2e32cf990f7d0c19408700d9d48ad6&quot;</ETag><Size>3678590</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-1-g9379a1d/tgz/tgz.sha256sum</Key><LastModified>2016-10-27T16:09:31.000Z</LastModified><ETag>&quot;ef3104e81c9ffc0dacec22191b46e5cc&quot;</ETag><Size>2102</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_darwin_386/converge</Key><LastModified>2016-11-09T21:57:22.000Z</LastModified><ETag>&quot;7c93d9520bbf26b8721c75baa7b71331&quot;</ETag><Size>10647152</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_darwin_386/converge.sha256sum</Key><LastModified>2016-11-09T21:57:23.000Z</LastModified><ETag>&quot;eb5757196388e92c1c3251dbbdcf4f67&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_darwin_amd64/converge</Key><LastModified>2016-11-09T21:57:23.000Z</LastModified><ETag>&quot;726a6101c4021da14c1a374d2c433865&quot;</ETag><Size>11945840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-09T21:57:23.000Z</LastModified><ETag>&quot;f92f8a87ebabbd2591dea2e0d8a94f26&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_freebsd_386/converge</Key><LastModified>2016-11-09T21:57:23.000Z</LastModified><ETag>&quot;7b8f3b77a2c1871e8a2c6c2b9f38e1bc&quot;</ETag><Size>9492256</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-09T21:57:24.000Z</LastModified><ETag>&quot;eeb7efda21e8b33b5319608e4a9938b8&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_freebsd_amd64/converge</Key><LastModified>2016-11-09T21:57:24.000Z</LastModified><ETag>&quot;a1f96db829457feea0f48c32faf1fa63&quot;</ETag><Size>10718592</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-09T21:57:24.000Z</LastModified><ETag>&quot;2914397c1294bc7433dec44e479a2c6c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_freebsd_arm/converge</Key><LastModified>2016-11-09T21:57:24.000Z</LastModified><ETag>&quot;bc0d1a5c55df8f51d383e1fb617e31d7&quot;</ETag><Size>9737064</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-09T21:57:24.000Z</LastModified><ETag>&quot;99a64c248f8da84f4e350ec744adebb5&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_386/converge</Key><LastModified>2016-11-09T21:57:24.000Z</LastModified><ETag>&quot;9ef10322112ade97d751757478e390f9&quot;</ETag><Size>9570304</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_386/converge.sha256sum</Key><LastModified>2016-11-09T21:57:25.000Z</LastModified><ETag>&quot;34c9ae88cfb176b886f5ac8292e74d96&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_amd64/converge</Key><LastModified>2016-11-09T21:57:25.000Z</LastModified><ETag>&quot;d2ff2782872a863220fea22c3b32b1b8&quot;</ETag><Size>10785760</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-09T21:57:26.000Z</LastModified><ETag>&quot;eac4f06338a4414d5d35d68b0e4129bb&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_arm/converge</Key><LastModified>2016-11-09T21:57:26.000Z</LastModified><ETag>&quot;9b8483351e404208955617cc4b9a06c4&quot;</ETag><Size>9729264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_arm/converge.sha256sum</Key><LastModified>2016-11-09T21:57:27.000Z</LastModified><ETag>&quot;03a73b912dcb501eae419f0f3dd0d8ee&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_arm64/converge</Key><LastModified>2016-11-09T21:57:27.000Z</LastModified><ETag>&quot;4a84481670ed0dd7d71928f7daf9f06a&quot;</ETag><Size>10959808</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-09T21:57:27.000Z</LastModified><ETag>&quot;69a0b875fb7822a17f8f7a294a072300&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_ppc64/converge</Key><LastModified>2016-11-09T21:57:27.000Z</LastModified><ETag>&quot;631d58dbb6c271fbea2a75a7b5e940d8&quot;</ETag><Size>11287552</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-09T21:57:29.000Z</LastModified><ETag>&quot;b7d1a240e62c8315f7f055949baf2ecc&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_ppc64le/converge</Key><LastModified>2016-11-09T21:57:29.000Z</LastModified><ETag>&quot;d94b2e8e6c339591f26fbfcaebc87db4&quot;</ETag><Size>11287520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-09T21:57:29.000Z</LastModified><ETag>&quot;5120f376942777ce58fbf71e324b5c84&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_solaris_amd64/converge</Key><LastModified>2016-11-09T21:57:30.000Z</LastModified><ETag>&quot;c9dd0c5dcd420dce338be3f71d2d61e5&quot;</ETag><Size>10648064</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/converge_0.3.0-103-gf6db1a7_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-09T21:57:30.000Z</LastModified><ETag>&quot;210c36ad2abf252ce3b10aefab07e321&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_darwin_386.tar.gz</Key><LastModified>2016-11-09T21:57:30.000Z</LastModified><ETag>&quot;5580db14e35ef493e0d9654362ef394d&quot;</ETag><Size>3762645</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_darwin_amd64.tar.gz</Key><LastModified>2016-11-09T21:57:30.000Z</LastModified><ETag>&quot;f94ebfbd9c668b2b8b07b7cf927da5ad&quot;</ETag><Size>3945600</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_freebsd_386.tar.gz</Key><LastModified>2016-11-09T21:57:30.000Z</LastModified><ETag>&quot;d7b216bb873e793982c9f2ce937f44a8&quot;</ETag><Size>3566014</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_freebsd_amd64.tar.gz</Key><LastModified>2016-11-09T21:57:30.000Z</LastModified><ETag>&quot;e26376a64b04ff2c79a6bef57c907cb8&quot;</ETag><Size>3751870</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_freebsd_arm.tar.gz</Key><LastModified>2016-11-09T21:57:31.000Z</LastModified><ETag>&quot;a36e3b345c4e582c0c81f896159cb1a6&quot;</ETag><Size>3467821</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_linux_386.tar.gz</Key><LastModified>2016-11-09T21:57:31.000Z</LastModified><ETag>&quot;b1fa9f53700a8a33f80c671b467d47b0&quot;</ETag><Size>3571104</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_linux_amd64.tar.gz</Key><LastModified>2016-11-09T21:57:31.000Z</LastModified><ETag>&quot;80514de9e72494c0239c6de0d3112ffe&quot;</ETag><Size>3780287</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_linux_arm.tar.gz</Key><LastModified>2016-11-09T21:57:31.000Z</LastModified><ETag>&quot;631843d6b7615af27e68477e99b9e084&quot;</ETag><Size>3470483</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_linux_arm64.tar.gz</Key><LastModified>2016-11-09T21:57:31.000Z</LastModified><ETag>&quot;b3f5fd48ede73314760c5477518edd93&quot;</ETag><Size>3498614</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_linux_ppc64.tar.gz</Key><LastModified>2016-11-09T21:57:31.000Z</LastModified><ETag>&quot;57b6d1481018225a594121f62a898bd0&quot;</ETag><Size>3512692</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_linux_ppc64le.tar.gz</Key><LastModified>2016-11-09T21:57:32.000Z</LastModified><ETag>&quot;96ce3a27ab802e70cbcc88b525dc7235&quot;</ETag><Size>3472162</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/converge_0.3.0-103-gf6db1a7_solaris_amd64.tar.gz</Key><LastModified>2016-11-09T21:57:32.000Z</LastModified><ETag>&quot;7e2442e2b6677a8bc976b330b51235aa&quot;</ETag><Size>3725582</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-103-gf6db1a7/tgz/tgz.sha256sum</Key><LastModified>2016-11-09T21:57:32.000Z</LastModified><ETag>&quot;2664bbb42a37ecea8fd6e52799a6a0d5&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_darwin_386/converge</Key><LastModified>2016-10-31T14:58:23.000Z</LastModified><ETag>&quot;6424da9a7387f508af939357a56b094d&quot;</ETag><Size>10518956</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_darwin_386/converge.sha256sum</Key><LastModified>2016-10-31T14:58:24.000Z</LastModified><ETag>&quot;d23ffe5eaeec4ef47c1e5fd14e6cbaf0&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_darwin_amd64/converge</Key><LastModified>2016-10-31T14:58:25.000Z</LastModified><ETag>&quot;689f979874c2514dfd2b0fe47f415d0a&quot;</ETag><Size>11800288</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_darwin_amd64/converge.sha256sum</Key><LastModified>2016-10-31T14:58:26.000Z</LastModified><ETag>&quot;9cd93043986dfddca81abfc8c4fe3500&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_freebsd_386/converge</Key><LastModified>2016-10-31T14:58:26.000Z</LastModified><ETag>&quot;88342532ebf4dfca6102609eb13abc87&quot;</ETag><Size>9377056</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_freebsd_386/converge.sha256sum</Key><LastModified>2016-10-31T14:58:27.000Z</LastModified><ETag>&quot;f442a8832e3e41742a37daf6bdb8d1d7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_freebsd_amd64/converge</Key><LastModified>2016-10-31T14:58:27.000Z</LastModified><ETag>&quot;d857108e98015ab7e14f51b39ef90984&quot;</ETag><Size>10590752</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-10-31T14:58:27.000Z</LastModified><ETag>&quot;858ca2fa627f3577e95712f862b2baf9&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_freebsd_arm/converge</Key><LastModified>2016-10-31T14:58:28.000Z</LastModified><ETag>&quot;e32bd262c866812ad9f463165dbe85b0&quot;</ETag><Size>9626056</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_freebsd_arm/converge.sha256sum</Key><LastModified>2016-10-31T14:58:28.000Z</LastModified><ETag>&quot;3e0a97220a9f8977817d3f497a565fde&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_386/converge</Key><LastModified>2016-10-31T14:58:28.000Z</LastModified><ETag>&quot;e0947e3c2b68a041cadb80b1cbb9d73b&quot;</ETag><Size>9455104</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_386/converge.sha256sum</Key><LastModified>2016-10-31T14:58:28.000Z</LastModified><ETag>&quot;bea1f4ae65b81737160c8ace320d3c2f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_amd64/converge</Key><LastModified>2016-10-31T14:58:28.000Z</LastModified><ETag>&quot;49307ddfdc60beedc13e8f28d86ed0fe&quot;</ETag><Size>10660160</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_amd64/converge.sha256sum</Key><LastModified>2016-10-31T14:58:29.000Z</LastModified><ETag>&quot;97151c842261169574a45444ee53de5a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_arm/converge</Key><LastModified>2016-10-31T14:58:29.000Z</LastModified><ETag>&quot;275f763885e99e2994ce535bb27918ab&quot;</ETag><Size>9622352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_arm/converge.sha256sum</Key><LastModified>2016-10-31T14:58:29.000Z</LastModified><ETag>&quot;a99a385560949e5ed97712770d6b41a9&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_arm64/converge</Key><LastModified>2016-10-31T14:58:30.000Z</LastModified><ETag>&quot;8c3122a35ede6e0b272400ea3dcde467&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_arm64/converge.sha256sum</Key><LastModified>2016-10-31T14:58:30.000Z</LastModified><ETag>&quot;a0d3cd01e73af6add8ad35ea5cae2766&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_ppc64/converge</Key><LastModified>2016-10-31T14:58:30.000Z</LastModified><ETag>&quot;dffc985c6a85e9ee8a530f55bb25e06b&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_ppc64/converge.sha256sum</Key><LastModified>2016-10-31T14:58:31.000Z</LastModified><ETag>&quot;38c4ebb792f1ae9c291215524c7b1a53&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_ppc64le/converge</Key><LastModified>2016-10-31T14:58:31.000Z</LastModified><ETag>&quot;cf1eb35ccd7766ac396ab2443302a098&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-10-31T14:58:32.000Z</LastModified><ETag>&quot;e84a0a9b0b7111d009c8360c8b2f56e7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_solaris_amd64/converge</Key><LastModified>2016-10-31T14:58:32.000Z</LastModified><ETag>&quot;26068bb9a1dd2e23d20a5376603c277e&quot;</ETag><Size>10520224</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/converge_0.3.0-11-gaa79ad9_solaris_amd64/converge.sha256sum</Key><LastModified>2016-10-31T14:58:32.000Z</LastModified><ETag>&quot;9276fa05dae0d004f40d8410b54a6488&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_darwin_386.tar.gz</Key><LastModified>2016-10-31T14:58:32.000Z</LastModified><ETag>&quot;750db585688a567a6dc8eab6069a5df1&quot;</ETag><Size>3725243</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_darwin_amd64.tar.gz</Key><LastModified>2016-10-31T14:58:33.000Z</LastModified><ETag>&quot;6cb27dd66ebb1a12c0318dce8c67be69&quot;</ETag><Size>3907127</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_freebsd_386.tar.gz</Key><LastModified>2016-10-31T14:58:33.000Z</LastModified><ETag>&quot;e737cf6381adc663b211cbb9789b291f&quot;</ETag><Size>3531819</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_freebsd_amd64.tar.gz</Key><LastModified>2016-10-31T14:58:33.000Z</LastModified><ETag>&quot;0cb17e16f5adb3c21fa99d5c06b082c8&quot;</ETag><Size>3714219</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_freebsd_arm.tar.gz</Key><LastModified>2016-10-31T14:58:34.000Z</LastModified><ETag>&quot;ffec462cb8c8fbfdded64b6b8c05ec5e&quot;</ETag><Size>3435013</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_linux_386.tar.gz</Key><LastModified>2016-10-31T14:58:34.000Z</LastModified><ETag>&quot;7be1dcc3f3c2a047335779457d20a694&quot;</ETag><Size>3538407</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_linux_amd64.tar.gz</Key><LastModified>2016-10-31T14:58:34.000Z</LastModified><ETag>&quot;316a20078c04f2450428ea6e1adbd976&quot;</ETag><Size>3744187</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_linux_arm.tar.gz</Key><LastModified>2016-10-31T14:58:34.000Z</LastModified><ETag>&quot;ef55a8377fd526e1814733d526faafab&quot;</ETag><Size>3438634</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_linux_arm64.tar.gz</Key><LastModified>2016-10-31T14:58:35.000Z</LastModified><ETag>&quot;f94ceb1e896f3fbe98807e2bec19cbc8&quot;</ETag><Size>3464813</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_linux_ppc64.tar.gz</Key><LastModified>2016-10-31T14:58:35.000Z</LastModified><ETag>&quot;6ee56dfb7daf772f47a19afffb7b2e17&quot;</ETag><Size>3478269</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_linux_ppc64le.tar.gz</Key><LastModified>2016-10-31T14:58:35.000Z</LastModified><ETag>&quot;651ed24752e1d0ed76c60514c85845f9&quot;</ETag><Size>3436044</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/converge_0.3.0-11-gaa79ad9_solaris_amd64.tar.gz</Key><LastModified>2016-10-31T14:58:35.000Z</LastModified><ETag>&quot;0d932f0d1a9d4130356bc54e4df693b1&quot;</ETag><Size>3687298</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-11-gaa79ad9/tgz/tgz.sha256sum</Key><LastModified>2016-10-31T14:58:35.000Z</LastModified><ETag>&quot;5e1cd14442c5144747514be19c72c531&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_darwin_386/converge</Key><LastModified>2016-10-31T15:19:52.000Z</LastModified><ETag>&quot;64c4ee7d4c9f9f5cfa6a05f5a7225c13&quot;</ETag><Size>10519180</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_darwin_386/converge.sha256sum</Key><LastModified>2016-10-31T15:19:53.000Z</LastModified><ETag>&quot;29aa8d1a889785f6bb5568dda6d3dcff&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_darwin_amd64/converge</Key><LastModified>2016-10-31T15:19:53.000Z</LastModified><ETag>&quot;c2763f26e870a23b269d6f79efa53eb8&quot;</ETag><Size>11800512</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_darwin_amd64/converge.sha256sum</Key><LastModified>2016-10-31T15:19:54.000Z</LastModified><ETag>&quot;38d2ec31ea64817cabf1b639b1f373e4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_freebsd_386/converge</Key><LastModified>2016-10-31T15:19:54.000Z</LastModified><ETag>&quot;06899c30dee28d9e2f6cb119e964e63c&quot;</ETag><Size>9381152</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_freebsd_386/converge.sha256sum</Key><LastModified>2016-10-31T15:19:55.000Z</LastModified><ETag>&quot;96ad14198f3edb3ac225b7ef19d417ad&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_freebsd_amd64/converge</Key><LastModified>2016-10-31T15:19:55.000Z</LastModified><ETag>&quot;cd701b644c2b6e1c6f0bb825cf68e4eb&quot;</ETag><Size>10586656</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-10-31T15:19:55.000Z</LastModified><ETag>&quot;bc09e8771a1e74fc8b78faf6b8c044a9&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_freebsd_arm/converge</Key><LastModified>2016-10-31T15:19:55.000Z</LastModified><ETag>&quot;3d8cccd5e8557e80dde7dcc7720f3086&quot;</ETag><Size>9626056</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_freebsd_arm/converge.sha256sum</Key><LastModified>2016-10-31T15:19:56.000Z</LastModified><ETag>&quot;341007e411516af6aac2f6b58e699a0a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_386/converge</Key><LastModified>2016-10-31T15:19:58.000Z</LastModified><ETag>&quot;443d3aee27eeca05c8b208daf16166f1&quot;</ETag><Size>9459200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_386/converge.sha256sum</Key><LastModified>2016-10-31T15:19:58.000Z</LastModified><ETag>&quot;3217bcdabebb75eb9d4e3566b98ad8f7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_amd64/converge</Key><LastModified>2016-10-31T15:19:58.000Z</LastModified><ETag>&quot;847435feb24b5ed18504230e0dc8c3d3&quot;</ETag><Size>10659712</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_amd64/converge.sha256sum</Key><LastModified>2016-10-31T15:19:59.000Z</LastModified><ETag>&quot;ac8bd41f793076e78d64ad5be20b35e2&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_arm/converge</Key><LastModified>2016-10-31T15:19:59.000Z</LastModified><ETag>&quot;c5b2e87ef06c604ef5b4b75c53c2bda7&quot;</ETag><Size>9622352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_arm/converge.sha256sum</Key><LastModified>2016-10-31T15:19:59.000Z</LastModified><ETag>&quot;c420ca4ca044e49355d81e3cda63c8c1&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_arm64/converge</Key><LastModified>2016-10-31T15:19:59.000Z</LastModified><ETag>&quot;ff50aa0d2255167c741f5a641280ade0&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_arm64/converge.sha256sum</Key><LastModified>2016-10-31T15:20:00.000Z</LastModified><ETag>&quot;912fbeae265a7906c5013377a35f8cc2&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_ppc64/converge</Key><LastModified>2016-10-31T15:20:00.000Z</LastModified><ETag>&quot;5781f438b3b28ed37a523b830872241c&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_ppc64/converge.sha256sum</Key><LastModified>2016-10-31T15:20:02.000Z</LastModified><ETag>&quot;df80917d298ff3fea058ffc385fba873&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_ppc64le/converge</Key><LastModified>2016-10-31T15:20:02.000Z</LastModified><ETag>&quot;d0200e2074a124c3b4f8cb6cd2a6692a&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-10-31T15:20:03.000Z</LastModified><ETag>&quot;bb8467e9bd59257810b159cc85552cfd&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_solaris_amd64/converge</Key><LastModified>2016-10-31T15:20:03.000Z</LastModified><ETag>&quot;038c5d1f5518cbc700d3716f7c91c2a4&quot;</ETag><Size>10520224</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/converge_0.3.0-15-ga7c5fb2_solaris_amd64/converge.sha256sum</Key><LastModified>2016-10-31T15:20:05.000Z</LastModified><ETag>&quot;842de6a770c4f487b3b9bc1e1ece2b54&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_darwin_386.tar.gz</Key><LastModified>2016-10-31T15:20:05.000Z</LastModified><ETag>&quot;51e74d8ca4f8be94b6df47871e5977b4&quot;</ETag><Size>3725773</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_darwin_amd64.tar.gz</Key><LastModified>2016-10-31T15:20:06.000Z</LastModified><ETag>&quot;eef7b5e9d7faa18fa623a87e0e4bb62e&quot;</ETag><Size>3906940</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_freebsd_386.tar.gz</Key><LastModified>2016-10-31T15:20:06.000Z</LastModified><ETag>&quot;2d0ef9eaea5fe601c8293d83f082fb88&quot;</ETag><Size>3532391</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_freebsd_amd64.tar.gz</Key><LastModified>2016-10-31T15:20:06.000Z</LastModified><ETag>&quot;52d38f75ebe7437656f8f46714565624&quot;</ETag><Size>3714763</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_freebsd_arm.tar.gz</Key><LastModified>2016-10-31T15:20:07.000Z</LastModified><ETag>&quot;54689a583788ab19f687ad910cdd4dca&quot;</ETag><Size>3435530</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_linux_386.tar.gz</Key><LastModified>2016-10-31T15:20:07.000Z</LastModified><ETag>&quot;e0243b84e47d56e7118742debd2eb698&quot;</ETag><Size>3538822</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_linux_amd64.tar.gz</Key><LastModified>2016-10-31T15:20:07.000Z</LastModified><ETag>&quot;29cf9335bc5ecced3967dc36a89103b2&quot;</ETag><Size>3744835</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_linux_arm.tar.gz</Key><LastModified>2016-10-31T15:20:08.000Z</LastModified><ETag>&quot;b2a481d2bdf0aa9da52ca747b076f4c1&quot;</ETag><Size>3438662</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_linux_arm64.tar.gz</Key><LastModified>2016-10-31T15:20:08.000Z</LastModified><ETag>&quot;140bb61c3cd95e84a00216082d6cc544&quot;</ETag><Size>3464259</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_linux_ppc64.tar.gz</Key><LastModified>2016-10-31T15:20:09.000Z</LastModified><ETag>&quot;df9df169463cf7c757e575e4cec71c25&quot;</ETag><Size>3478070</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_linux_ppc64le.tar.gz</Key><LastModified>2016-10-31T15:20:09.000Z</LastModified><ETag>&quot;5c339c02123cecb8a3b50c3802d3e369&quot;</ETag><Size>3436853</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/converge_0.3.0-15-ga7c5fb2_solaris_amd64.tar.gz</Key><LastModified>2016-10-31T15:20:09.000Z</LastModified><ETag>&quot;5cb82f1821cec25b1df68a8c1c6f9ccb&quot;</ETag><Size>3687139</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-15-ga7c5fb2/tgz/tgz.sha256sum</Key><LastModified>2016-10-31T15:20:10.000Z</LastModified><ETag>&quot;0e8ca18d6e07ec1ed946c6ba45809f0f&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_darwin_386/converge</Key><LastModified>2016-11-10T20:19:39.000Z</LastModified><ETag>&quot;60455733537dc61f1e3bedb8147cf846&quot;</ETag><Size>10738736</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_darwin_386/converge.sha256sum</Key><LastModified>2016-11-10T20:19:40.000Z</LastModified><ETag>&quot;37828032c38fdf50f0372ba720123e91&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_darwin_amd64/converge</Key><LastModified>2016-11-10T20:19:40.000Z</LastModified><ETag>&quot;86df3390fc36eabb66c22df0f138e30d&quot;</ETag><Size>12041968</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-10T20:19:40.000Z</LastModified><ETag>&quot;0592bf752808c301f714518119f4dda2&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_freebsd_386/converge</Key><LastModified>2016-11-10T20:19:40.000Z</LastModified><ETag>&quot;92f9ec8ecefc3023e0b8670ed39836fe&quot;</ETag><Size>9574496</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-10T20:19:41.000Z</LastModified><ETag>&quot;4343b8e197aff35e298f9c52337b6a40&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_freebsd_amd64/converge</Key><LastModified>2016-11-10T20:19:41.000Z</LastModified><ETag>&quot;7d39c0888b078ed352d6c89339e7c392&quot;</ETag><Size>10801120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-10T20:19:41.000Z</LastModified><ETag>&quot;6578970142efffd176c2300d8b152327&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_freebsd_arm/converge</Key><LastModified>2016-11-10T20:19:41.000Z</LastModified><ETag>&quot;cbb65584bfb7e9b277377a0ec3bca969&quot;</ETag><Size>9811088</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-10T20:19:42.000Z</LastModified><ETag>&quot;a24bfce31f9546573a034176ee701e0e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_386/converge</Key><LastModified>2016-11-10T20:19:42.000Z</LastModified><ETag>&quot;35ece2abc50fa441bf585e4ef19b19d2&quot;</ETag><Size>9652544</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_386/converge.sha256sum</Key><LastModified>2016-11-10T20:19:43.000Z</LastModified><ETag>&quot;44a333e0ea8d3b0f1261200f52954309&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_amd64/converge</Key><LastModified>2016-11-10T20:20:09.000Z</LastModified><ETag>&quot;4926f86fc0192aa08e04faa57e864675&quot;</ETag><Size>10876256</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-10T20:20:09.000Z</LastModified><ETag>&quot;8012c4e1ded02af167418f50214f02da&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_arm/converge</Key><LastModified>2016-11-10T20:20:10.000Z</LastModified><ETag>&quot;39182c83da2adddf9a8c2d2b2d10207c&quot;</ETag><Size>9811480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_arm/converge.sha256sum</Key><LastModified>2016-11-10T20:20:10.000Z</LastModified><ETag>&quot;884414535d5fa218cb124c89fe96686b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_arm64/converge</Key><LastModified>2016-11-10T20:20:10.000Z</LastModified><ETag>&quot;38a84a82b943743ea7a0ea68b18d203b&quot;</ETag><Size>11091488</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-10T20:20:11.000Z</LastModified><ETag>&quot;d91b80d68097723b6450e05e4eafb4a7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_ppc64/converge</Key><LastModified>2016-11-10T20:20:11.000Z</LastModified><ETag>&quot;ac28e2544ef325f0c6643b296d03fd7d&quot;</ETag><Size>11419232</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-10T20:20:14.000Z</LastModified><ETag>&quot;0af56d5ff5553bb0d30acab53eacf7a6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_ppc64le/converge</Key><LastModified>2016-11-10T20:20:14.000Z</LastModified><ETag>&quot;447a3c1d0b6b816036fa57240cc13666&quot;</ETag><Size>11419200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-10T20:20:15.000Z</LastModified><ETag>&quot;254ce4c2d614f411db2a061b301af62e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_solaris_amd64/converge</Key><LastModified>2016-11-10T20:20:15.000Z</LastModified><ETag>&quot;f880ff0a7d2c52ab4dd2c6ccd5ce51b3&quot;</ETag><Size>10738784</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/converge_0.3.0-153-g11beb43_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-10T20:20:15.000Z</LastModified><ETag>&quot;46e93d70f7048638210fe0c78e98c2cd&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_darwin_386.tar.gz</Key><LastModified>2016-11-10T20:20:15.000Z</LastModified><ETag>&quot;330532a3183a5a92eabe97013b3c6343&quot;</ETag><Size>3793122</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_darwin_amd64.tar.gz</Key><LastModified>2016-11-10T20:20:15.000Z</LastModified><ETag>&quot;c36eb57d49fa34040a61a1916609fbd2&quot;</ETag><Size>3975597</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_freebsd_386.tar.gz</Key><LastModified>2016-11-10T20:20:16.000Z</LastModified><ETag>&quot;ea8e8eac426389008bb8d150569a704e&quot;</ETag><Size>3595392</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_freebsd_amd64.tar.gz</Key><LastModified>2016-11-10T20:20:16.000Z</LastModified><ETag>&quot;d4655cbf1ae605c9924146230e793568&quot;</ETag><Size>3780021</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_freebsd_arm.tar.gz</Key><LastModified>2016-11-10T20:20:17.000Z</LastModified><ETag>&quot;9d138060e36e37ed718c1484edfe7e6b&quot;</ETag><Size>3496771</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_linux_386.tar.gz</Key><LastModified>2016-11-10T20:20:17.000Z</LastModified><ETag>&quot;436214ba4117ebde9089ece5a5f0afe8&quot;</ETag><Size>3602493</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_linux_amd64.tar.gz</Key><LastModified>2016-11-10T20:20:17.000Z</LastModified><ETag>&quot;a8b2cdb589abdc595e2e390ad22cc00c&quot;</ETag><Size>3812037</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_linux_arm.tar.gz</Key><LastModified>2016-11-10T20:20:17.000Z</LastModified><ETag>&quot;8aaef33962d650532d4b043b0e1d87ff&quot;</ETag><Size>3499392</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_linux_arm64.tar.gz</Key><LastModified>2016-11-10T20:20:18.000Z</LastModified><ETag>&quot;5ac4b2d6ad375a694e7632084ca505ed&quot;</ETag><Size>3527875</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_linux_ppc64.tar.gz</Key><LastModified>2016-11-10T20:20:18.000Z</LastModified><ETag>&quot;f78de21afc738f71266822e37a7c008a&quot;</ETag><Size>3543554</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_linux_ppc64le.tar.gz</Key><LastModified>2016-11-10T20:20:18.000Z</LastModified><ETag>&quot;4bc59edb0f9cbb264df2283097be4012&quot;</ETag><Size>3499182</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/converge_0.3.0-153-g11beb43_solaris_amd64.tar.gz</Key><LastModified>2016-11-10T20:20:19.000Z</LastModified><ETag>&quot;8e103a138912a260b411555f63a92081&quot;</ETag><Size>3754902</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-153-g11beb43/tgz/tgz.sha256sum</Key><LastModified>2016-11-10T20:20:19.000Z</LastModified><ETag>&quot;bed66e4e2098dd728eacaaf80206145b&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_darwin_386/converge</Key><LastModified>2016-10-31T20:33:14.000Z</LastModified><ETag>&quot;2867a739c3dfe9a1f1f59c795fa5bc73&quot;</ETag><Size>10519240</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_darwin_386/converge.sha256sum</Key><LastModified>2016-10-31T20:33:15.000Z</LastModified><ETag>&quot;595cb309582d57d8169b9465cdfba94c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_darwin_amd64/converge</Key><LastModified>2016-10-31T20:33:15.000Z</LastModified><ETag>&quot;560a3eb92904da38e0ae11081698c0ad&quot;</ETag><Size>11804672</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_darwin_amd64/converge.sha256sum</Key><LastModified>2016-10-31T20:33:16.000Z</LastModified><ETag>&quot;54c05e7bdf794b6f52e29b6a0a36281e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_freebsd_386/converge</Key><LastModified>2016-10-31T20:33:16.000Z</LastModified><ETag>&quot;3cbfb1f1fab136461bf8d711a7f87d54&quot;</ETag><Size>9385248</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_freebsd_386/converge.sha256sum</Key><LastModified>2016-10-31T20:33:16.000Z</LastModified><ETag>&quot;08cd58ead76b13690117a89f94f3dd32&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_freebsd_amd64/converge</Key><LastModified>2016-10-31T20:33:16.000Z</LastModified><ETag>&quot;b87a907017f5b5edc6ae9d4430c61cb5&quot;</ETag><Size>10590752</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-10-31T20:33:17.000Z</LastModified><ETag>&quot;162f08f52004f7435b1ba29791cabaec&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_freebsd_arm/converge</Key><LastModified>2016-10-31T20:33:17.000Z</LastModified><ETag>&quot;886a3c67a6c5e5b0c40333d3f335c555&quot;</ETag><Size>9626056</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_freebsd_arm/converge.sha256sum</Key><LastModified>2016-10-31T20:33:18.000Z</LastModified><ETag>&quot;b03d51b9d5e0332a91d427e72e6f5657&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_386/converge</Key><LastModified>2016-10-31T20:33:18.000Z</LastModified><ETag>&quot;462d72b8d2c10058339673fcbc516d08&quot;</ETag><Size>9459200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_386/converge.sha256sum</Key><LastModified>2016-10-31T20:33:18.000Z</LastModified><ETag>&quot;d866cd7c7011bb6ccff21c65409b49f3&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_amd64/converge</Key><LastModified>2016-10-31T20:33:18.000Z</LastModified><ETag>&quot;1a7afcc4dd3edd02e73d531018f1c886&quot;</ETag><Size>10660608</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_amd64/converge.sha256sum</Key><LastModified>2016-10-31T20:33:19.000Z</LastModified><ETag>&quot;ad96285e2a145b66c6056e28b6241f73&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_arm/converge</Key><LastModified>2016-10-31T20:33:19.000Z</LastModified><ETag>&quot;91c25c5aabb9a42b409340040e3fa6b7&quot;</ETag><Size>9622352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_arm/converge.sha256sum</Key><LastModified>2016-10-31T20:33:19.000Z</LastModified><ETag>&quot;a17e2aba2a164508b2cdbc70eaa7607b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_arm64/converge</Key><LastModified>2016-10-31T20:33:19.000Z</LastModified><ETag>&quot;3df77f19a0e44eced15d270a5c54ff89&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_arm64/converge.sha256sum</Key><LastModified>2016-10-31T20:33:20.000Z</LastModified><ETag>&quot;b1828a98cde5be8a66538c84b7f3cd02&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_ppc64/converge</Key><LastModified>2016-10-31T20:33:20.000Z</LastModified><ETag>&quot;4917315774f4263e08de439b06bc4b0e&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_ppc64/converge.sha256sum</Key><LastModified>2016-10-31T20:33:21.000Z</LastModified><ETag>&quot;e42e51a78684535e4cd88442231d3630&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_ppc64le/converge</Key><LastModified>2016-10-31T20:33:21.000Z</LastModified><ETag>&quot;d036fd936fd4c207f017710c0f3b6e7c&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-10-31T20:33:21.000Z</LastModified><ETag>&quot;83f3e01df776f169c1022f8102d7f903&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_solaris_amd64/converge</Key><LastModified>2016-10-31T20:33:21.000Z</LastModified><ETag>&quot;c513c422c73d9981f3e2453fdb53d3ca&quot;</ETag><Size>10524320</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/converge_0.3.0-17-gc1e030f_solaris_amd64/converge.sha256sum</Key><LastModified>2016-10-31T20:33:22.000Z</LastModified><ETag>&quot;33fac34b445417f6775c955545c482ce&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_darwin_386.tar.gz</Key><LastModified>2016-10-31T20:33:22.000Z</LastModified><ETag>&quot;5b00419a5cb8bead34f4058291e45916&quot;</ETag><Size>3726001</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_darwin_amd64.tar.gz</Key><LastModified>2016-10-31T20:33:22.000Z</LastModified><ETag>&quot;70008d26246d5c555bc0ee8c572d3f25&quot;</ETag><Size>3907467</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_freebsd_386.tar.gz</Key><LastModified>2016-10-31T20:33:23.000Z</LastModified><ETag>&quot;edd1a505e9fd1ee64474c97f3b79b794&quot;</ETag><Size>3532641</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_freebsd_amd64.tar.gz</Key><LastModified>2016-10-31T20:33:23.000Z</LastModified><ETag>&quot;5c15d43c43f970b130f1c01bfa9afa1a&quot;</ETag><Size>3715485</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_freebsd_arm.tar.gz</Key><LastModified>2016-10-31T20:33:24.000Z</LastModified><ETag>&quot;067dcb5843ec75e9ef3fce5065c5c51a&quot;</ETag><Size>3435362</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_linux_386.tar.gz</Key><LastModified>2016-10-31T20:33:24.000Z</LastModified><ETag>&quot;628f3d0bba2e3da54a8aa2c4db020098&quot;</ETag><Size>3539727</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_linux_amd64.tar.gz</Key><LastModified>2016-10-31T20:33:24.000Z</LastModified><ETag>&quot;edc0dd8566c9f8fbb001d0db41fab802&quot;</ETag><Size>3745528</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_linux_arm.tar.gz</Key><LastModified>2016-10-31T20:33:25.000Z</LastModified><ETag>&quot;d39a71d820c16f80cdb5048cc9ecd3a2&quot;</ETag><Size>3439100</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_linux_arm64.tar.gz</Key><LastModified>2016-10-31T20:33:25.000Z</LastModified><ETag>&quot;ad8367b7c07f25e0e5bb9d128bb5d1e1&quot;</ETag><Size>3465662</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_linux_ppc64.tar.gz</Key><LastModified>2016-10-31T20:33:26.000Z</LastModified><ETag>&quot;98a99635986db7048dbf2e1061c8a9ca&quot;</ETag><Size>3478962</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_linux_ppc64le.tar.gz</Key><LastModified>2016-10-31T20:33:26.000Z</LastModified><ETag>&quot;9da7ba3e3c911e163948913f1c65b69b&quot;</ETag><Size>3437430</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/converge_0.3.0-17-gc1e030f_solaris_amd64.tar.gz</Key><LastModified>2016-10-31T20:33:26.000Z</LastModified><ETag>&quot;5e3af6a5734c42f448c71d468e6f756b&quot;</ETag><Size>3687658</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-17-gc1e030f/tgz/tgz.sha256sum</Key><LastModified>2016-10-31T20:33:27.000Z</LastModified><ETag>&quot;cac329fef1a4c3f32db136481ed66549&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_darwin_386/converge</Key><LastModified>2016-11-14T17:36:09.000Z</LastModified><ETag>&quot;42d5940ea78df9b4cb037fbedbc8672d&quot;</ETag><Size>10756200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_darwin_386/converge.sha256sum</Key><LastModified>2016-11-14T17:36:09.000Z</LastModified><ETag>&quot;729115d950800e838e925046eda028f7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_darwin_amd64/converge</Key><LastModified>2016-11-14T17:36:09.000Z</LastModified><ETag>&quot;b52f0386bd9b4ffb2835c1a4e1bcc791&quot;</ETag><Size>12059488</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-14T17:36:10.000Z</LastModified><ETag>&quot;a83c7394975abc929efa1bfbed86dd58&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_freebsd_386/converge</Key><LastModified>2016-11-14T17:36:11.000Z</LastModified><ETag>&quot;60ca19e64c6f3484783976226565af95&quot;</ETag><Size>9590912</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-14T17:36:11.000Z</LastModified><ETag>&quot;f65b4d9d5dc063c684d062556fae9763&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_freebsd_amd64/converge</Key><LastModified>2016-11-14T17:36:11.000Z</LastModified><ETag>&quot;f499b536755f7f85d034a2b209000858&quot;</ETag><Size>10817536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-14T17:36:12.000Z</LastModified><ETag>&quot;ee1f3ae05fd2cf9aa3eec11875b2d9b6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_freebsd_arm/converge</Key><LastModified>2016-11-14T17:36:12.000Z</LastModified><ETag>&quot;30f63297c98fe0a022781cc51162b00e&quot;</ETag><Size>9827480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-14T17:36:12.000Z</LastModified><ETag>&quot;9069a9bcea1c0c9c46c272ccc73df711&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_386/converge</Key><LastModified>2016-11-14T17:36:12.000Z</LastModified><ETag>&quot;48e4666a09caf6696e7dec0c59a597ff&quot;</ETag><Size>9673056</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_386/converge.sha256sum</Key><LastModified>2016-11-14T17:36:13.000Z</LastModified><ETag>&quot;f395c60cc8629d95435acdb9255a5464&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_amd64/converge</Key><LastModified>2016-11-14T17:36:13.000Z</LastModified><ETag>&quot;2149e6c771b1ee36f1e085b71f429c77&quot;</ETag><Size>10892960</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-14T17:36:13.000Z</LastModified><ETag>&quot;1214df975fc53e5d3ce84046bdde1e58&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_arm/converge</Key><LastModified>2016-11-14T17:36:13.000Z</LastModified><ETag>&quot;7454274517885c7b88aec5448d785b2b&quot;</ETag><Size>9827872</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_arm/converge.sha256sum</Key><LastModified>2016-11-14T17:36:15.000Z</LastModified><ETag>&quot;b0197895e7d47a417c510021c4315eac&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_arm64/converge</Key><LastModified>2016-11-14T17:36:15.000Z</LastModified><ETag>&quot;823c0deacac11bb07723e3353987eb8c&quot;</ETag><Size>11091520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-14T17:36:17.000Z</LastModified><ETag>&quot;87e46b046b1aa013fff5d8ac431d9321&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_ppc64/converge</Key><LastModified>2016-11-14T17:36:17.000Z</LastModified><ETag>&quot;16a8bb6eb7a53445cecd10e9a1e3a264&quot;</ETag><Size>11419264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-14T17:36:18.000Z</LastModified><ETag>&quot;0cf8d00b5833431d636fae4c4ca3b748&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_ppc64le/converge</Key><LastModified>2016-11-14T17:36:18.000Z</LastModified><ETag>&quot;24077f8bd6bbdaf288ca6e55113eabce&quot;</ETag><Size>11419232</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-14T17:36:18.000Z</LastModified><ETag>&quot;222c08cd1b1e966addbc02a3a8f50e8a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_solaris_amd64/converge</Key><LastModified>2016-11-14T17:36:18.000Z</LastModified><ETag>&quot;b5639718cfcd32fae6b034c59ebce4b7&quot;</ETag><Size>10755200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/converge_0.3.0-173-g13fc6c3_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-14T17:36:19.000Z</LastModified><ETag>&quot;75aee1cf123ba68621569347ca58395b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_darwin_386.tar.gz</Key><LastModified>2016-11-14T17:36:19.000Z</LastModified><ETag>&quot;a9d8bf04488625bb0501b9c65247ae1b&quot;</ETag><Size>3800411</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_darwin_amd64.tar.gz</Key><LastModified>2016-11-14T17:36:19.000Z</LastModified><ETag>&quot;8f17deb7c1f62527b53e839c70c441d3&quot;</ETag><Size>3982844</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_freebsd_386.tar.gz</Key><LastModified>2016-11-14T17:36:19.000Z</LastModified><ETag>&quot;5bbbae76f6ddfda1ea0fc51c4fc494ef&quot;</ETag><Size>3600760</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_freebsd_amd64.tar.gz</Key><LastModified>2016-11-14T17:36:20.000Z</LastModified><ETag>&quot;8c1f3e9c4677b5f596ee016b73f476ad&quot;</ETag><Size>3786900</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_freebsd_arm.tar.gz</Key><LastModified>2016-11-14T17:36:20.000Z</LastModified><ETag>&quot;1a180d6939ee350d10262d76346f8f25&quot;</ETag><Size>3502186</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_linux_386.tar.gz</Key><LastModified>2016-11-14T17:36:20.000Z</LastModified><ETag>&quot;b9ae16fe892fef3fd99742948fcc7aa9&quot;</ETag><Size>3607423</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_linux_amd64.tar.gz</Key><LastModified>2016-11-14T17:36:20.000Z</LastModified><ETag>&quot;ee4ffcac12822babf18161d2e7f642e5&quot;</ETag><Size>3817562</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_linux_arm.tar.gz</Key><LastModified>2016-11-14T17:36:21.000Z</LastModified><ETag>&quot;4a1f9ed7ee4ed59cf003d199672b07a0&quot;</ETag><Size>3504249</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_linux_arm64.tar.gz</Key><LastModified>2016-11-14T17:36:21.000Z</LastModified><ETag>&quot;1425f532444458775c4d10af17420b08&quot;</ETag><Size>3533073</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_linux_ppc64.tar.gz</Key><LastModified>2016-11-14T17:36:21.000Z</LastModified><ETag>&quot;0fec9050d04b0d876c7c8e1d7f2c8533&quot;</ETag><Size>3548778</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_linux_ppc64le.tar.gz</Key><LastModified>2016-11-14T17:36:21.000Z</LastModified><ETag>&quot;6180aa00b67eef1fd6f971de06d20d06&quot;</ETag><Size>3502813</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/converge_0.3.0-173-g13fc6c3_solaris_amd64.tar.gz</Key><LastModified>2016-11-14T17:36:22.000Z</LastModified><ETag>&quot;96cb1b0171a3905057c6f42a46426c3e&quot;</ETag><Size>3762701</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-173-g13fc6c3/tgz/tgz.sha256sum</Key><LastModified>2016-11-14T17:36:22.000Z</LastModified><ETag>&quot;9c3fe69a121bf03e053a87a146f05a21&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_darwin_386/converge</Key><LastModified>2016-11-01T20:47:25.000Z</LastModified><ETag>&quot;6b9f4d57c061e78c74b8ed1cd4533099&quot;</ETag><Size>10523532</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_darwin_386/converge.sha256sum</Key><LastModified>2016-11-01T20:47:25.000Z</LastModified><ETag>&quot;52374c866975b2674a8b17de5d83282e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_darwin_amd64/converge</Key><LastModified>2016-11-01T20:47:25.000Z</LastModified><ETag>&quot;43d144d3e8eab8318bfb99e502c9fe7d&quot;</ETag><Size>11804896</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-01T20:47:26.000Z</LastModified><ETag>&quot;b3bbceed1632628e5b20b38ffa9bb50c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_freebsd_386/converge</Key><LastModified>2016-11-01T20:47:26.000Z</LastModified><ETag>&quot;09c12ead07ac4ea01c679a42875bea55&quot;</ETag><Size>9385248</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-01T20:47:26.000Z</LastModified><ETag>&quot;52441e5f8167ec72cbc41c104f61d41e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_freebsd_amd64/converge</Key><LastModified>2016-11-01T20:47:26.000Z</LastModified><ETag>&quot;20efd47271faf374ce2de68cf10f7996&quot;</ETag><Size>10590752</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-01T20:47:29.000Z</LastModified><ETag>&quot;2b2d2a0c34ffd9a60bf60263a2b4a3ad&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_freebsd_arm/converge</Key><LastModified>2016-11-01T20:47:30.000Z</LastModified><ETag>&quot;beb9fc443a041ae2888146efd9c35742&quot;</ETag><Size>9630152</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-01T20:47:30.000Z</LastModified><ETag>&quot;8c8870c992c9fe9d504a540009373bd8&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_386/converge</Key><LastModified>2016-11-01T20:47:30.000Z</LastModified><ETag>&quot;fb0ce392d995b1849b4eb785baf9d996&quot;</ETag><Size>9459200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_386/converge.sha256sum</Key><LastModified>2016-11-01T20:47:31.000Z</LastModified><ETag>&quot;faef41faca30fd9da5fa0d315eff73f5&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_amd64/converge</Key><LastModified>2016-11-01T20:47:31.000Z</LastModified><ETag>&quot;e6324dd57e87fd30ec599f7b0112c94d&quot;</ETag><Size>10664192</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-01T20:47:31.000Z</LastModified><ETag>&quot;7b2fa1624d205bd0623141740ac4ec4c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_arm/converge</Key><LastModified>2016-11-01T20:47:31.000Z</LastModified><ETag>&quot;9f6aaab80ef38dbdf63ed9e2e278c2cc&quot;</ETag><Size>9626448</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_arm/converge.sha256sum</Key><LastModified>2016-11-01T20:47:32.000Z</LastModified><ETag>&quot;da52153aeacb437e8213b89df8b2a29a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_arm64/converge</Key><LastModified>2016-11-01T20:47:32.000Z</LastModified><ETag>&quot;cc46abde13982839b1439a72dca1da1e&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-01T20:47:33.000Z</LastModified><ETag>&quot;64b84cd0026b5240d8d9604c46f15a0b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_ppc64/converge</Key><LastModified>2016-11-01T20:47:33.000Z</LastModified><ETag>&quot;ff83921cf6e45a57c748fa77f5c9b3f9&quot;</ETag><Size>11221120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-01T20:47:33.000Z</LastModified><ETag>&quot;3506984546991819c5fe8b0b0a010714&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_ppc64le/converge</Key><LastModified>2016-11-01T20:47:33.000Z</LastModified><ETag>&quot;7a09bd102548dc432aafc76ca6cac123&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-01T20:47:34.000Z</LastModified><ETag>&quot;565ac1bd05d2a7dca63d5abe3a4c9fbd&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_solaris_amd64/converge</Key><LastModified>2016-11-01T20:47:34.000Z</LastModified><ETag>&quot;a43ce3e9cab161a82099b3fcffb92fab&quot;</ETag><Size>10528416</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/converge_0.3.0-19-gae78477_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-01T20:47:34.000Z</LastModified><ETag>&quot;d149197dc55c55b4f3c45e05acd1eb83&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_darwin_386.tar.gz</Key><LastModified>2016-11-01T20:47:34.000Z</LastModified><ETag>&quot;74e2052d8c64ff9006b72782155e0878&quot;</ETag><Size>3726627</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_darwin_amd64.tar.gz</Key><LastModified>2016-11-01T20:47:34.000Z</LastModified><ETag>&quot;06068373237c7cb8aa2b26602d5c6462&quot;</ETag><Size>3908130</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_freebsd_386.tar.gz</Key><LastModified>2016-11-01T20:47:35.000Z</LastModified><ETag>&quot;38b3494b578851eb661e262e5cebf376&quot;</ETag><Size>3533431</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_freebsd_amd64.tar.gz</Key><LastModified>2016-11-01T20:47:35.000Z</LastModified><ETag>&quot;2c23672c843ea23c95d71806823da234&quot;</ETag><Size>3716098</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_freebsd_arm.tar.gz</Key><LastModified>2016-11-01T20:47:36.000Z</LastModified><ETag>&quot;f6d45cdff0e840dbe497ff5890c33afd&quot;</ETag><Size>3438428</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_linux_386.tar.gz</Key><LastModified>2016-11-01T20:47:36.000Z</LastModified><ETag>&quot;ef4775405b05a93ad61706cc38bb1b57&quot;</ETag><Size>3540560</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_linux_amd64.tar.gz</Key><LastModified>2016-11-01T20:47:36.000Z</LastModified><ETag>&quot;3021e76563f070bf645482f4894965f2&quot;</ETag><Size>3746230</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_linux_arm.tar.gz</Key><LastModified>2016-11-01T20:47:37.000Z</LastModified><ETag>&quot;f60ed2851dc149bd910ad55643368233&quot;</ETag><Size>3438562</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_linux_arm64.tar.gz</Key><LastModified>2016-11-01T20:47:37.000Z</LastModified><ETag>&quot;f04169c8fd32d2b408d3d234f174c764&quot;</ETag><Size>3466371</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_linux_ppc64.tar.gz</Key><LastModified>2016-11-01T20:47:37.000Z</LastModified><ETag>&quot;6418596328064cdd2b67b1d0aec57e7c&quot;</ETag><Size>3480393</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_linux_ppc64le.tar.gz</Key><LastModified>2016-11-01T20:47:37.000Z</LastModified><ETag>&quot;bae93b286e3cf8a72668521c988772ea&quot;</ETag><Size>3438798</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/converge_0.3.0-19-gae78477_solaris_amd64.tar.gz</Key><LastModified>2016-11-01T20:47:38.000Z</LastModified><ETag>&quot;59ad12eeb076083f0e598e66498f9f1a&quot;</ETag><Size>3690266</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-19-gae78477/tgz/tgz.sha256sum</Key><LastModified>2016-11-01T20:47:38.000Z</LastModified><ETag>&quot;22325f2363db51fc63df413ae7c32bf6&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_darwin_386/converge</Key><LastModified>2016-11-14T18:17:36.000Z</LastModified><ETag>&quot;1ee50399c271992ce9fa1ca50eceec92&quot;</ETag><Size>10756200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_darwin_386/converge.sha256sum</Key><LastModified>2016-11-14T18:17:36.000Z</LastModified><ETag>&quot;81437707d56f9d9865ca8530132649aa&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_darwin_amd64/converge</Key><LastModified>2016-11-14T18:17:36.000Z</LastModified><ETag>&quot;e9d2e3fc7627a958f9a43733d55c12cf&quot;</ETag><Size>12059472</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-14T18:17:37.000Z</LastModified><ETag>&quot;5452d4063c70e4eca22391789e013b98&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_freebsd_386/converge</Key><LastModified>2016-11-14T18:17:37.000Z</LastModified><ETag>&quot;44f91341d3dc156e7bc4fced00621ff2&quot;</ETag><Size>9590912</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-14T18:17:39.000Z</LastModified><ETag>&quot;f21495a823f2a3ebdf0cd5d0fa1fd27c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_freebsd_amd64/converge</Key><LastModified>2016-11-14T18:17:39.000Z</LastModified><ETag>&quot;c4a947aa06872fe0fc328c9f092d6877&quot;</ETag><Size>10817536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-14T18:17:39.000Z</LastModified><ETag>&quot;ac6a18946ffcb2dd74acbf6a049260e3&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_freebsd_arm/converge</Key><LastModified>2016-11-14T18:17:39.000Z</LastModified><ETag>&quot;cf7fa6424d83029cee3452a1f852b46b&quot;</ETag><Size>9827480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-14T18:17:40.000Z</LastModified><ETag>&quot;651a5a3285e45532707988e28f8be830&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_386/converge</Key><LastModified>2016-11-14T18:17:40.000Z</LastModified><ETag>&quot;a2aeef214677ffdf6d47b35d2c91fd52&quot;</ETag><Size>9668960</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_386/converge.sha256sum</Key><LastModified>2016-11-14T18:17:40.000Z</LastModified><ETag>&quot;49b7c3fd2600c54f59735c4cb79b3e3f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_amd64/converge</Key><LastModified>2016-11-14T18:17:40.000Z</LastModified><ETag>&quot;c35f90ffa49453a64a513c54394aa322&quot;</ETag><Size>10893216</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-14T18:17:40.000Z</LastModified><ETag>&quot;63af4fbb86ceced23ce665ef39e4ef0d&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_arm/converge</Key><LastModified>2016-11-14T18:17:41.000Z</LastModified><ETag>&quot;cf78d694ef692b299f7dee749c3b0a77&quot;</ETag><Size>9827872</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_arm/converge.sha256sum</Key><LastModified>2016-11-14T18:17:41.000Z</LastModified><ETag>&quot;5cfab2ba4e82c21bce1e1f2fe3c1bda7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_arm64/converge</Key><LastModified>2016-11-14T18:17:41.000Z</LastModified><ETag>&quot;fccf0863ea12342dfbea85512f339699&quot;</ETag><Size>11091520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-14T18:17:41.000Z</LastModified><ETag>&quot;13ae91c1fdcc014ead63a57ee7660e18&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_ppc64/converge</Key><LastModified>2016-11-14T18:17:42.000Z</LastModified><ETag>&quot;fd6ea9204f38a8433331e2ce834ad42d&quot;</ETag><Size>11419264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-14T18:17:42.000Z</LastModified><ETag>&quot;7542176ccf41cd6e499eb690d95a2d67&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_ppc64le/converge</Key><LastModified>2016-11-14T18:17:42.000Z</LastModified><ETag>&quot;2350037d1f4b25c5fa43919391235a35&quot;</ETag><Size>11419232</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-14T18:17:44.000Z</LastModified><ETag>&quot;d2dec0c6858de83467fe4ab07e53774b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_solaris_amd64/converge</Key><LastModified>2016-11-14T18:17:44.000Z</LastModified><ETag>&quot;5379275e44e3f7f6e586b1b845a0c03c&quot;</ETag><Size>10759296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/converge_0.3.0-192-g4ff647a_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-14T18:17:44.000Z</LastModified><ETag>&quot;c5eee4c76b52c9dfb48a6bc58984b85c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_darwin_386.tar.gz</Key><LastModified>2016-11-14T18:17:44.000Z</LastModified><ETag>&quot;d1a4661cae684aa4623fa80f5fb05c90&quot;</ETag><Size>3800544</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_darwin_amd64.tar.gz</Key><LastModified>2016-11-14T18:17:44.000Z</LastModified><ETag>&quot;3bd96d4f362510ad7f650ba329b10507&quot;</ETag><Size>3982769</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_freebsd_386.tar.gz</Key><LastModified>2016-11-14T18:17:45.000Z</LastModified><ETag>&quot;7d9d6fcec8a3c9de88b6b44c36992758&quot;</ETag><Size>3600519</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_freebsd_amd64.tar.gz</Key><LastModified>2016-11-14T18:17:45.000Z</LastModified><ETag>&quot;08d61dfc41428a59e7a7818491a6978b&quot;</ETag><Size>3786684</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_freebsd_arm.tar.gz</Key><LastModified>2016-11-14T18:17:45.000Z</LastModified><ETag>&quot;e3c7a29ddccfd51c6b1e6b48e48bc2a2&quot;</ETag><Size>3502355</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_linux_386.tar.gz</Key><LastModified>2016-11-14T18:17:45.000Z</LastModified><ETag>&quot;67a290502d88da9f098dbe6a77d5320e&quot;</ETag><Size>3607476</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_linux_amd64.tar.gz</Key><LastModified>2016-11-14T18:17:45.000Z</LastModified><ETag>&quot;e99fdfa4f94aaba4fedff2d75ffd6635&quot;</ETag><Size>3817431</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_linux_arm.tar.gz</Key><LastModified>2016-11-14T18:17:46.000Z</LastModified><ETag>&quot;5564b72d2470c39eb416b682ada8fbe0&quot;</ETag><Size>3504273</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_linux_arm64.tar.gz</Key><LastModified>2016-11-14T18:17:46.000Z</LastModified><ETag>&quot;f25c937df6213853c127fe8b712ce69b&quot;</ETag><Size>3534033</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_linux_ppc64.tar.gz</Key><LastModified>2016-11-14T18:17:46.000Z</LastModified><ETag>&quot;981d62b908433d545056be0353aefd0f&quot;</ETag><Size>3546386</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_linux_ppc64le.tar.gz</Key><LastModified>2016-11-14T18:17:46.000Z</LastModified><ETag>&quot;89c8c3fd4cc50d51c3b42faa592a87f4&quot;</ETag><Size>3502925</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/converge_0.3.0-192-g4ff647a_solaris_amd64.tar.gz</Key><LastModified>2016-11-14T18:17:46.000Z</LastModified><ETag>&quot;08cb18496b98582d92a58dc45c9fca7b&quot;</ETag><Size>3762433</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-192-g4ff647a/tgz/tgz.sha256sum</Key><LastModified>2016-11-14T18:17:47.000Z</LastModified><ETag>&quot;bcfa8576ff069e718abce3e6825e213e&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_darwin_386/converge</Key><LastModified>2016-11-14T20:34:17.000Z</LastModified><ETag>&quot;fdd75a1449d855665a10dcf5d294d408&quot;</ETag><Size>10756200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_darwin_386/converge.sha256sum</Key><LastModified>2016-11-14T20:34:17.000Z</LastModified><ETag>&quot;fb922bc90dfa258c5f78efc14dde1312&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_darwin_amd64/converge</Key><LastModified>2016-11-14T20:34:17.000Z</LastModified><ETag>&quot;49c2f5f913a17eb706fa0a3264c14c32&quot;</ETag><Size>12059472</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-14T20:34:18.000Z</LastModified><ETag>&quot;699ab6aef595c4d87b24b47e67aaa77a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_freebsd_386/converge</Key><LastModified>2016-11-14T20:34:18.000Z</LastModified><ETag>&quot;c72a1d5689f8707034092ed9d8901353&quot;</ETag><Size>9590912</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-14T20:34:18.000Z</LastModified><ETag>&quot;53584d5b72f8dbdf9979a80b10486a56&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_freebsd_amd64/converge</Key><LastModified>2016-11-14T20:34:19.000Z</LastModified><ETag>&quot;482cc7f2fa0c154ed68a6ccd080876ca&quot;</ETag><Size>10817536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-14T20:34:19.000Z</LastModified><ETag>&quot;5e9b5886e02a37d40e64c149ee64fb79&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_freebsd_arm/converge</Key><LastModified>2016-11-14T20:34:19.000Z</LastModified><ETag>&quot;bbf75c00e40fada476d9404486c9f05e&quot;</ETag><Size>9827480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-14T20:34:19.000Z</LastModified><ETag>&quot;7a3480063d79108eb2a6687f9019ff82&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_386/converge</Key><LastModified>2016-11-14T20:34:19.000Z</LastModified><ETag>&quot;c29f79dbc8ebf1417edde67e088fdb96&quot;</ETag><Size>9668960</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_386/converge.sha256sum</Key><LastModified>2016-11-14T20:34:20.000Z</LastModified><ETag>&quot;a04620681e2dc954012d63bbd74950c9&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_amd64/converge</Key><LastModified>2016-11-14T20:34:20.000Z</LastModified><ETag>&quot;45d8add51ba98a0d13801f4f3e28e46c&quot;</ETag><Size>10893216</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-14T20:34:22.000Z</LastModified><ETag>&quot;5b65eb784301587844378306ed8b414c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_arm/converge</Key><LastModified>2016-11-14T20:34:22.000Z</LastModified><ETag>&quot;7e65aca247aff1006e90cad3b8259957&quot;</ETag><Size>9827872</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_arm/converge.sha256sum</Key><LastModified>2016-11-14T20:34:23.000Z</LastModified><ETag>&quot;bb4b6bc0e639aa04bf2f1a99abfc3fb1&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_arm64/converge</Key><LastModified>2016-11-14T20:34:23.000Z</LastModified><ETag>&quot;e952319e4baaea8e072a60849081c0b2&quot;</ETag><Size>11091520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-14T20:34:23.000Z</LastModified><ETag>&quot;bded0c3809eb4c159450dffac1c148ff&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_ppc64/converge</Key><LastModified>2016-11-14T20:34:23.000Z</LastModified><ETag>&quot;c9c4073c6441721819b73030cf474346&quot;</ETag><Size>11419264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-14T20:34:24.000Z</LastModified><ETag>&quot;0c12aaa549af498dbfd2f1cab785f86e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_ppc64le/converge</Key><LastModified>2016-11-14T20:34:24.000Z</LastModified><ETag>&quot;76b5af8d663c05e1f4eebc5879319814&quot;</ETag><Size>11419232</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-14T20:34:25.000Z</LastModified><ETag>&quot;549664733a94b40a4aaad5621854b0e4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_solaris_amd64/converge</Key><LastModified>2016-11-14T20:34:25.000Z</LastModified><ETag>&quot;0675df0c61752e2814a382f957e7bfb8&quot;</ETag><Size>10759296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/converge_0.3.0-194-g131d29b_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-14T20:34:26.000Z</LastModified><ETag>&quot;692b1c2b8ec74d07c672783545885425&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_darwin_386.tar.gz</Key><LastModified>2016-11-14T20:34:26.000Z</LastModified><ETag>&quot;cc7ee6c76dd5f9caef5d7944562b6f04&quot;</ETag><Size>3800550</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_darwin_amd64.tar.gz</Key><LastModified>2016-11-14T20:34:27.000Z</LastModified><ETag>&quot;d4924a05fa7a6a17ce4facd4cfa14068&quot;</ETag><Size>3982767</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_freebsd_386.tar.gz</Key><LastModified>2016-11-14T20:34:27.000Z</LastModified><ETag>&quot;6f6077e952e330d366c91e4db1c648a6&quot;</ETag><Size>3600540</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_freebsd_amd64.tar.gz</Key><LastModified>2016-11-14T20:34:27.000Z</LastModified><ETag>&quot;dcf3977529f842be3654c15acbad405e&quot;</ETag><Size>3786685</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_freebsd_arm.tar.gz</Key><LastModified>2016-11-14T20:34:27.000Z</LastModified><ETag>&quot;33c46dddb99883536d30ac139842b1c1&quot;</ETag><Size>3502352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_linux_386.tar.gz</Key><LastModified>2016-11-14T20:34:27.000Z</LastModified><ETag>&quot;88b6f0d464fe8d9ec14b552c65e78bb9&quot;</ETag><Size>3607477</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_linux_amd64.tar.gz</Key><LastModified>2016-11-14T20:34:28.000Z</LastModified><ETag>&quot;c41f907672a65440710d54fd460b837b&quot;</ETag><Size>3817449</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_linux_arm.tar.gz</Key><LastModified>2016-11-14T20:34:28.000Z</LastModified><ETag>&quot;a11ac7830262c9f7af893cd1525f9ab4&quot;</ETag><Size>3504289</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_linux_arm64.tar.gz</Key><LastModified>2016-11-14T20:34:28.000Z</LastModified><ETag>&quot;6f02cec7215c646563f403beddad2beb&quot;</ETag><Size>3534014</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_linux_ppc64.tar.gz</Key><LastModified>2016-11-14T20:34:28.000Z</LastModified><ETag>&quot;8335a8710a65fd3da918d22d4fe8d498&quot;</ETag><Size>3546082</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_linux_ppc64le.tar.gz</Key><LastModified>2016-11-14T20:34:29.000Z</LastModified><ETag>&quot;e1d86ae2a0a5d50214799421c9e49d3b&quot;</ETag><Size>3502927</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/converge_0.3.0-194-g131d29b_solaris_amd64.tar.gz</Key><LastModified>2016-11-14T20:34:29.000Z</LastModified><ETag>&quot;79b8f38e97ffc23a911aea7cd4d4a032&quot;</ETag><Size>3762427</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-194-g131d29b/tgz/tgz.sha256sum</Key><LastModified>2016-11-14T20:34:29.000Z</LastModified><ETag>&quot;dc22909a6df1b5956c9613c2651e9faf&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_darwin_386/converge</Key><LastModified>2016-11-15T15:09:27.000Z</LastModified><ETag>&quot;e04206ad7d0c2676157b1c62ed9b3b13&quot;</ETag><Size>10756200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_darwin_386/converge.sha256sum</Key><LastModified>2016-11-15T15:09:28.000Z</LastModified><ETag>&quot;54fdf17c1dc56d16dfcb0da70619e08a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_darwin_amd64/converge</Key><LastModified>2016-11-15T15:09:28.000Z</LastModified><ETag>&quot;4a3891306c8194ba02e4d3e96712057d&quot;</ETag><Size>12059472</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-15T15:09:28.000Z</LastModified><ETag>&quot;b51f282da698af4552159e96e400d147&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_freebsd_386/converge</Key><LastModified>2016-11-15T15:09:28.000Z</LastModified><ETag>&quot;5ecd81f088b469f05f66bb8c5bf26c6f&quot;</ETag><Size>9590912</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-15T15:09:28.000Z</LastModified><ETag>&quot;00d6a475d70334a175683c39de7413d6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_freebsd_amd64/converge</Key><LastModified>2016-11-15T15:09:28.000Z</LastModified><ETag>&quot;0815ee1c4a24614952cf9db771efcaaf&quot;</ETag><Size>10817536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-15T15:09:29.000Z</LastModified><ETag>&quot;589d3669a3b4b0de20d73302dbf2d43b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_freebsd_arm/converge</Key><LastModified>2016-11-15T15:09:29.000Z</LastModified><ETag>&quot;c05968046aa21eb5d8b9750c2d71d6aa&quot;</ETag><Size>9827480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-15T15:09:29.000Z</LastModified><ETag>&quot;8a0f9da50d9c2aa6561f2a976890ecab&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_386/converge</Key><LastModified>2016-11-15T15:09:29.000Z</LastModified><ETag>&quot;b6f912d7f22f3beba1936920e438192c&quot;</ETag><Size>9668960</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_386/converge.sha256sum</Key><LastModified>2016-11-15T15:09:31.000Z</LastModified><ETag>&quot;95625418d9f530b3bd03d68a5fcfbfc2&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_amd64/converge</Key><LastModified>2016-11-15T15:09:31.000Z</LastModified><ETag>&quot;233f4421e64f167d2b997b5f84409a70&quot;</ETag><Size>10893248</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-15T15:09:31.000Z</LastModified><ETag>&quot;031c9051858b4e6194a7559478e58f3a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_arm/converge</Key><LastModified>2016-11-15T15:09:31.000Z</LastModified><ETag>&quot;27c109231551184a7bfb6416c5c0117f&quot;</ETag><Size>9827872</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_arm/converge.sha256sum</Key><LastModified>2016-11-15T15:09:31.000Z</LastModified><ETag>&quot;b8c18e4fa421acb24817ece8888e6f9a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_arm64/converge</Key><LastModified>2016-11-15T15:09:32.000Z</LastModified><ETag>&quot;d0155ada33b39b6a688876556fd77661&quot;</ETag><Size>11091520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-15T15:09:32.000Z</LastModified><ETag>&quot;cdb2f6debed3f761ce6242c765e15396&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_ppc64/converge</Key><LastModified>2016-11-15T15:09:32.000Z</LastModified><ETag>&quot;c484f37f571735dfa97884a07909fb4e&quot;</ETag><Size>11419264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-15T15:09:33.000Z</LastModified><ETag>&quot;5b6c2172b3861ffb63636d8e6f9b7734&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_ppc64le/converge</Key><LastModified>2016-11-15T15:09:33.000Z</LastModified><ETag>&quot;b97d2e354406fd57e12ab87d9ea63ac9&quot;</ETag><Size>11419232</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-15T15:09:33.000Z</LastModified><ETag>&quot;4daddc79e6e23c3de02d6fadb6ba4165&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_solaris_amd64/converge</Key><LastModified>2016-11-15T15:09:33.000Z</LastModified><ETag>&quot;15a44d15335d89cf6a2aeba25fa7fe8d&quot;</ETag><Size>10759296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/converge_0.3.0-196-ga908214_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-15T15:09:34.000Z</LastModified><ETag>&quot;4083bb6d7c484e0e5f4dde248b0a0d26&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_darwin_386.tar.gz</Key><LastModified>2016-11-15T15:09:34.000Z</LastModified><ETag>&quot;5e07b7149f3dc4131c08404e7a34b6f2&quot;</ETag><Size>3800461</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_darwin_amd64.tar.gz</Key><LastModified>2016-11-15T15:09:34.000Z</LastModified><ETag>&quot;1746663a9b572566031bad29ff7a9e08&quot;</ETag><Size>3982836</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_freebsd_386.tar.gz</Key><LastModified>2016-11-15T15:09:34.000Z</LastModified><ETag>&quot;3359d63801b9f977fc1863cf9a776bdd&quot;</ETag><Size>3600685</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_freebsd_amd64.tar.gz</Key><LastModified>2016-11-15T15:09:36.000Z</LastModified><ETag>&quot;701fe67408b11504b8d3fe0169b93850&quot;</ETag><Size>3786704</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_freebsd_arm.tar.gz</Key><LastModified>2016-11-15T15:09:36.000Z</LastModified><ETag>&quot;dd2700f482e7b78532f8312c535957f3&quot;</ETag><Size>3502308</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_linux_386.tar.gz</Key><LastModified>2016-11-15T15:09:36.000Z</LastModified><ETag>&quot;6f83f8b522ed0823b78d5144d32f9db7&quot;</ETag><Size>3607611</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_linux_amd64.tar.gz</Key><LastModified>2016-11-15T15:09:36.000Z</LastModified><ETag>&quot;4b6e355dc41de239fa3babe9048d6109&quot;</ETag><Size>3817513</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_linux_arm.tar.gz</Key><LastModified>2016-11-15T15:09:37.000Z</LastModified><ETag>&quot;6559c44fbd1d6a298520ee6184a1d5de&quot;</ETag><Size>3504515</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_linux_arm64.tar.gz</Key><LastModified>2016-11-15T15:09:37.000Z</LastModified><ETag>&quot;4b1163248f76b8ff0dc2d16a8d5fc686&quot;</ETag><Size>3533770</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_linux_ppc64.tar.gz</Key><LastModified>2016-11-15T15:09:37.000Z</LastModified><ETag>&quot;4a80096d1b69e688b51ac05c9164c1fd&quot;</ETag><Size>3545834</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_linux_ppc64le.tar.gz</Key><LastModified>2016-11-15T15:09:37.000Z</LastModified><ETag>&quot;a920bd5f048e69cc3356885f24139fdf&quot;</ETag><Size>3502920</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/converge_0.3.0-196-ga908214_solaris_amd64.tar.gz</Key><LastModified>2016-11-15T15:09:37.000Z</LastModified><ETag>&quot;6fb7d3061d0c55c2751cab9edb22c789&quot;</ETag><Size>3762487</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-ga908214/tgz/tgz.sha256sum</Key><LastModified>2016-11-15T15:09:38.000Z</LastModified><ETag>&quot;bb7ee3279787f88344ac17dbce2ff72e&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_darwin_386/converge</Key><LastModified>2016-11-15T20:59:32.000Z</LastModified><ETag>&quot;bcf0dfde7277eaaccf5f8d8f020d1d33&quot;</ETag><Size>10756200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_darwin_386/converge.sha256sum</Key><LastModified>2016-11-15T20:59:34.000Z</LastModified><ETag>&quot;c4882659d4fe8b8049a4281bf461d00c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_darwin_amd64/converge</Key><LastModified>2016-11-15T20:59:34.000Z</LastModified><ETag>&quot;dbdf6b975011dee109ad9c055cdae702&quot;</ETag><Size>12059472</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-15T20:59:34.000Z</LastModified><ETag>&quot;0a47e8005e91a911dd81dd91c7960370&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_freebsd_386/converge</Key><LastModified>2016-11-15T20:59:34.000Z</LastModified><ETag>&quot;f3fa7ae3246baf4c481c02c8632b95f5&quot;</ETag><Size>9590912</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-15T20:59:34.000Z</LastModified><ETag>&quot;22f2bd236e49f192cc139a08435c9390&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_freebsd_amd64/converge</Key><LastModified>2016-11-15T20:59:34.000Z</LastModified><ETag>&quot;5889aab46f44c6758df9e0e9f9b92da2&quot;</ETag><Size>10817536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-15T20:59:35.000Z</LastModified><ETag>&quot;22dd2ed7de646c3721a8f75d0c15b6da&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_freebsd_arm/converge</Key><LastModified>2016-11-15T20:59:35.000Z</LastModified><ETag>&quot;cd655d04837e2a1bc8c2613cd1bc2719&quot;</ETag><Size>9827480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-15T20:59:36.000Z</LastModified><ETag>&quot;190bd611b14da3475e46740129756bc4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_386/converge</Key><LastModified>2016-11-15T20:59:36.000Z</LastModified><ETag>&quot;91f3f7a2615cc44edcc9a9beec5c5c23&quot;</ETag><Size>9668960</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_386/converge.sha256sum</Key><LastModified>2016-11-15T20:59:37.000Z</LastModified><ETag>&quot;6ddbf01b9afcdc85ede3dc868c4a9cb9&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_amd64/converge</Key><LastModified>2016-11-15T20:59:37.000Z</LastModified><ETag>&quot;0973e180525c45a5f1b62685f6625862&quot;</ETag><Size>10893248</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-15T20:59:37.000Z</LastModified><ETag>&quot;42d872ebd18f5bd62962da1bb14c181c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_arm/converge</Key><LastModified>2016-11-15T20:59:37.000Z</LastModified><ETag>&quot;a0de86acfb63c900abe8d46d7758af98&quot;</ETag><Size>9827872</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_arm/converge.sha256sum</Key><LastModified>2016-11-15T20:59:38.000Z</LastModified><ETag>&quot;415904d85f508fede76e765b97d53051&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_arm64/converge</Key><LastModified>2016-11-15T20:59:38.000Z</LastModified><ETag>&quot;14485e53f12c75606742a22bfb9bfa02&quot;</ETag><Size>11091520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-15T20:59:39.000Z</LastModified><ETag>&quot;b081c33cf76b9758f156eba8ea89e39b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_ppc64/converge</Key><LastModified>2016-11-15T20:59:39.000Z</LastModified><ETag>&quot;605bed59b12f9249e6831f82864e8d7f&quot;</ETag><Size>11419264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-15T20:59:39.000Z</LastModified><ETag>&quot;1d4cd2a984d141a918d8fae3ddac0a7b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_ppc64le/converge</Key><LastModified>2016-11-15T20:59:39.000Z</LastModified><ETag>&quot;e781ca7dd28d7a54f4ca7f45c49ea2a5&quot;</ETag><Size>11419232</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-15T20:59:40.000Z</LastModified><ETag>&quot;13b71fa48c6b405195cf51fc5fca79d6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_solaris_amd64/converge</Key><LastModified>2016-11-15T20:59:40.000Z</LastModified><ETag>&quot;cb18593e4fd7a39bac0132993aa91491&quot;</ETag><Size>10759296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/converge_0.3.0-196-gcc298ba_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-15T20:59:40.000Z</LastModified><ETag>&quot;ab017cfee7cbf14ac01e6828060fb9ab&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_darwin_386.tar.gz</Key><LastModified>2016-11-15T20:59:40.000Z</LastModified><ETag>&quot;27e219a8044a4537d515f888e1ebe6fa&quot;</ETag><Size>3800451</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_darwin_amd64.tar.gz</Key><LastModified>2016-11-15T20:59:41.000Z</LastModified><ETag>&quot;d369e3132241b5f1a3eb9f3d059131b2&quot;</ETag><Size>3982838</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_freebsd_386.tar.gz</Key><LastModified>2016-11-15T20:59:41.000Z</LastModified><ETag>&quot;c6196ad0b7aaa1f36682569fb8e58c25&quot;</ETag><Size>3600606</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_freebsd_amd64.tar.gz</Key><LastModified>2016-11-15T20:59:41.000Z</LastModified><ETag>&quot;d823d7a700eea16b75ef2d927fb4a65f&quot;</ETag><Size>3786661</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_freebsd_arm.tar.gz</Key><LastModified>2016-11-15T20:59:42.000Z</LastModified><ETag>&quot;60e3b632637a315b19511dd1d33339aa&quot;</ETag><Size>3502274</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_linux_386.tar.gz</Key><LastModified>2016-11-15T20:59:42.000Z</LastModified><ETag>&quot;9fcaed08839b22c5f85ded295be34305&quot;</ETag><Size>3607611</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_linux_amd64.tar.gz</Key><LastModified>2016-11-15T20:59:42.000Z</LastModified><ETag>&quot;0f652ac64f8912636938ab0e6bb99201&quot;</ETag><Size>3817509</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_linux_arm.tar.gz</Key><LastModified>2016-11-15T20:59:42.000Z</LastModified><ETag>&quot;e98c254636cbeca067a90f484339983c&quot;</ETag><Size>3504448</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_linux_arm64.tar.gz</Key><LastModified>2016-11-15T20:59:43.000Z</LastModified><ETag>&quot;f7f0b9a7df9627d38cd37b4ed36268d8&quot;</ETag><Size>3533839</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_linux_ppc64.tar.gz</Key><LastModified>2016-11-15T20:59:43.000Z</LastModified><ETag>&quot;6cfa61bcc61bf9cd233283df0e4d1247&quot;</ETag><Size>3545834</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_linux_ppc64le.tar.gz</Key><LastModified>2016-11-15T20:59:43.000Z</LastModified><ETag>&quot;3a27634d77f5052269a319bef6b078ee&quot;</ETag><Size>3502937</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/converge_0.3.0-196-gcc298ba_solaris_amd64.tar.gz</Key><LastModified>2016-11-15T20:59:43.000Z</LastModified><ETag>&quot;08587897b50bac3e6573fe6c55b063a2&quot;</ETag><Size>3762473</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-196-gcc298ba/tgz/tgz.sha256sum</Key><LastModified>2016-11-15T20:59:44.000Z</LastModified><ETag>&quot;e849c0eef075c516215ab1601c47ad54&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_darwin_386/converge</Key><LastModified>2016-11-15T21:27:11.000Z</LastModified><ETag>&quot;292a6d9e5add109127745c82e2109c66&quot;</ETag><Size>10756200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_darwin_386/converge.sha256sum</Key><LastModified>2016-11-15T21:27:12.000Z</LastModified><ETag>&quot;38e3990d412bbaf62d64826c97174fd3&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_darwin_amd64/converge</Key><LastModified>2016-11-15T21:27:12.000Z</LastModified><ETag>&quot;b4059d82f60e47db99d0222f9eb4b52f&quot;</ETag><Size>12059472</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-15T21:27:12.000Z</LastModified><ETag>&quot;7031d9a29d23ade9af3ae18630ff2fa2&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_freebsd_386/converge</Key><LastModified>2016-11-15T21:27:12.000Z</LastModified><ETag>&quot;1f32d13e459c03d6b04861ddcb82be46&quot;</ETag><Size>9590912</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-15T21:27:13.000Z</LastModified><ETag>&quot;23a4d7457ee2fc9dff43c010018515b8&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_freebsd_amd64/converge</Key><LastModified>2016-11-15T21:27:13.000Z</LastModified><ETag>&quot;f7f5d3cbd80e5001e1bb4e0d10fcb9bf&quot;</ETag><Size>10817536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-15T21:27:13.000Z</LastModified><ETag>&quot;20faaa6165703eb52d76b5c411af8a4c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_freebsd_arm/converge</Key><LastModified>2016-11-15T21:27:13.000Z</LastModified><ETag>&quot;d66bc912452977502e78ce0fb669cfe6&quot;</ETag><Size>9827480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-15T21:27:13.000Z</LastModified><ETag>&quot;9df68d64df8793b8e2ad586e95a9b318&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_386/converge</Key><LastModified>2016-11-15T21:27:14.000Z</LastModified><ETag>&quot;0763afd28284ef0af5f517455b5452d7&quot;</ETag><Size>9668960</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_386/converge.sha256sum</Key><LastModified>2016-11-15T21:27:14.000Z</LastModified><ETag>&quot;3527bfb720172aa6cac663c1e631a9e7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_amd64/converge</Key><LastModified>2016-11-15T21:27:14.000Z</LastModified><ETag>&quot;b3cfae68c1e05b73b4977aef1813755d&quot;</ETag><Size>10893248</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-15T21:27:14.000Z</LastModified><ETag>&quot;2445c4c49d88f692c746ff5de4c4760d&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_arm/converge</Key><LastModified>2016-11-15T21:27:15.000Z</LastModified><ETag>&quot;9a3e64d36acdde83d1504721797323d2&quot;</ETag><Size>9827872</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_arm/converge.sha256sum</Key><LastModified>2016-11-15T21:27:15.000Z</LastModified><ETag>&quot;c99d6d35e0ce678ea9be277a666586ef&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_arm64/converge</Key><LastModified>2016-11-15T21:27:15.000Z</LastModified><ETag>&quot;845f2b48568977365dc633ba3d3ed189&quot;</ETag><Size>11091520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-15T21:27:16.000Z</LastModified><ETag>&quot;0be16cf6ee8475c1447df2d0dde964cd&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_ppc64/converge</Key><LastModified>2016-11-15T21:27:16.000Z</LastModified><ETag>&quot;bffbb93ff9cb29e26956664f46d8d59c&quot;</ETag><Size>11419264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-15T21:27:16.000Z</LastModified><ETag>&quot;76eaffe9e17380baf6060d0de27970f4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_ppc64le/converge</Key><LastModified>2016-11-15T21:27:16.000Z</LastModified><ETag>&quot;9a63bfe306f81a2b671a2cbd88b4fb40&quot;</ETag><Size>11419232</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-15T21:27:16.000Z</LastModified><ETag>&quot;0db5748106dacde389fdec93623a909b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_solaris_amd64/converge</Key><LastModified>2016-11-15T21:27:17.000Z</LastModified><ETag>&quot;1f22235d8b0597c105df86168fad8a57&quot;</ETag><Size>10759296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/converge_0.3.0-197-g0cea86f_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-15T21:27:17.000Z</LastModified><ETag>&quot;67b765a971209c9aec095e160a1ef6ac&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_darwin_386.tar.gz</Key><LastModified>2016-11-15T21:27:17.000Z</LastModified><ETag>&quot;f13992e5342dc13d2a59e9944cb81089&quot;</ETag><Size>3800459</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_darwin_amd64.tar.gz</Key><LastModified>2016-11-15T21:27:17.000Z</LastModified><ETag>&quot;9fe6a6071cbe48f83fa1631a6028aa6c&quot;</ETag><Size>3982855</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_freebsd_386.tar.gz</Key><LastModified>2016-11-15T21:27:17.000Z</LastModified><ETag>&quot;e725e1692f4fed496e93f70bbf8ebb36&quot;</ETag><Size>3600686</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_freebsd_amd64.tar.gz</Key><LastModified>2016-11-15T21:27:18.000Z</LastModified><ETag>&quot;f888f0a36840eff9282c823f7ce1f897&quot;</ETag><Size>3786699</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_freebsd_arm.tar.gz</Key><LastModified>2016-11-15T21:27:18.000Z</LastModified><ETag>&quot;1974105cd360c46d4de6190aa7b8aa4d&quot;</ETag><Size>3502313</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_linux_386.tar.gz</Key><LastModified>2016-11-15T21:27:18.000Z</LastModified><ETag>&quot;080d6a32b53d50754f7202c9fcba3aa1&quot;</ETag><Size>3607674</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_linux_amd64.tar.gz</Key><LastModified>2016-11-15T21:27:18.000Z</LastModified><ETag>&quot;ebf0281f6a3e81498dfbb8c6202681b3&quot;</ETag><Size>3817530</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_linux_arm.tar.gz</Key><LastModified>2016-11-15T21:27:18.000Z</LastModified><ETag>&quot;60f546b320c48f1c242102b8895628b4&quot;</ETag><Size>3504516</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_linux_arm64.tar.gz</Key><LastModified>2016-11-15T21:27:19.000Z</LastModified><ETag>&quot;1c523cb1eb86c382d2b679e7498f0a11&quot;</ETag><Size>3533841</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_linux_ppc64.tar.gz</Key><LastModified>2016-11-15T21:27:19.000Z</LastModified><ETag>&quot;07579143a142c0976ace9f0ca835e06b&quot;</ETag><Size>3545834</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_linux_ppc64le.tar.gz</Key><LastModified>2016-11-15T21:27:19.000Z</LastModified><ETag>&quot;4cacd6f6f2fa356ab40f7671916b47e6&quot;</ETag><Size>3502954</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/converge_0.3.0-197-g0cea86f_solaris_amd64.tar.gz</Key><LastModified>2016-11-15T21:27:19.000Z</LastModified><ETag>&quot;ad7be3e38735c345557beb588dec95a3&quot;</ETag><Size>3762474</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-197-g0cea86f/tgz/tgz.sha256sum</Key><LastModified>2016-11-15T21:27:20.000Z</LastModified><ETag>&quot;d745d87559043d73c5e7810b42e0c2e3&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_darwin_386/converge</Key><LastModified>2016-11-15T21:32:07.000Z</LastModified><ETag>&quot;029e59f4303b05303f9db7ce3a62548b&quot;</ETag><Size>10756200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_darwin_386/converge.sha256sum</Key><LastModified>2016-11-15T21:32:09.000Z</LastModified><ETag>&quot;a2597c9ed124a14c18090b329c224f89&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_darwin_amd64/converge</Key><LastModified>2016-11-15T21:32:09.000Z</LastModified><ETag>&quot;d391d4fe74a36a43f866406e4df33b30&quot;</ETag><Size>12059472</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-15T21:32:10.000Z</LastModified><ETag>&quot;28822b249dbe6f250152386866812fc7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_freebsd_386/converge</Key><LastModified>2016-11-15T21:32:10.000Z</LastModified><ETag>&quot;8a73443833e95e2d9ddc2535e89d88ba&quot;</ETag><Size>9590912</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-15T21:32:10.000Z</LastModified><ETag>&quot;7804444b2f6bfe139f6753b7d89855f3&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_freebsd_amd64/converge</Key><LastModified>2016-11-15T21:32:10.000Z</LastModified><ETag>&quot;6d46dca02629dd6ecc1526b8ce6d91aa&quot;</ETag><Size>10817536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-15T21:32:11.000Z</LastModified><ETag>&quot;b032feeff794a24947e764b11dbc7b34&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_freebsd_arm/converge</Key><LastModified>2016-11-15T21:32:11.000Z</LastModified><ETag>&quot;76224488beb9a3befca0220a2c8ee709&quot;</ETag><Size>9827480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-15T21:32:11.000Z</LastModified><ETag>&quot;fc707f2e2a5e4cfa4a8e1645dec4a38f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_386/converge</Key><LastModified>2016-11-15T21:32:11.000Z</LastModified><ETag>&quot;0d3914a2333eaa34af777a59333f3677&quot;</ETag><Size>9668960</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_386/converge.sha256sum</Key><LastModified>2016-11-15T21:32:12.000Z</LastModified><ETag>&quot;06b8137f574f6a5ee19398d8c09afa2b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_amd64/converge</Key><LastModified>2016-11-15T21:32:12.000Z</LastModified><ETag>&quot;74e99ffa269939547d30bab64846eb0c&quot;</ETag><Size>10893248</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-15T21:32:14.000Z</LastModified><ETag>&quot;76c693d4f8aa4d30de7d45b97c3db9af&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_arm/converge</Key><LastModified>2016-11-15T21:32:14.000Z</LastModified><ETag>&quot;bc1de0b8b1ccf9d30565440e06e2834d&quot;</ETag><Size>9827872</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_arm/converge.sha256sum</Key><LastModified>2016-11-15T21:32:14.000Z</LastModified><ETag>&quot;d0c5b277137144f75c2c85a81f86d4b1&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_arm64/converge</Key><LastModified>2016-11-15T21:32:14.000Z</LastModified><ETag>&quot;c82824340511cbc3884f8f8074cc8331&quot;</ETag><Size>11091520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-15T21:32:15.000Z</LastModified><ETag>&quot;bc58489c9223c615eaed699b4dec67e8&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_ppc64/converge</Key><LastModified>2016-11-15T21:32:15.000Z</LastModified><ETag>&quot;a38344442315f2d56036ab2f958b6c59&quot;</ETag><Size>11419264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-15T21:32:15.000Z</LastModified><ETag>&quot;17396440d2ec52ebd54de5db3f3bf2f0&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_ppc64le/converge</Key><LastModified>2016-11-15T21:32:15.000Z</LastModified><ETag>&quot;1e570cc36f2e246b4f832f4c56f19177&quot;</ETag><Size>11419232</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-15T21:32:16.000Z</LastModified><ETag>&quot;b41f18037c693a1a4961354d761680f5&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_solaris_amd64/converge</Key><LastModified>2016-11-15T21:32:16.000Z</LastModified><ETag>&quot;9ecac33fc947f6df583deaca3039ef80&quot;</ETag><Size>10759296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/converge_0.3.0-198-g7e7c642_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-15T21:32:16.000Z</LastModified><ETag>&quot;f978dbaad9443ea8e3ca5ecf5a32600a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_darwin_386.tar.gz</Key><LastModified>2016-11-15T21:32:16.000Z</LastModified><ETag>&quot;403ee9cd491f3582a516fb32400c7db0&quot;</ETag><Size>3800460</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_darwin_amd64.tar.gz</Key><LastModified>2016-11-15T21:32:16.000Z</LastModified><ETag>&quot;88fd3791a0642761f3f705d5f2d87a43&quot;</ETag><Size>3982854</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_freebsd_386.tar.gz</Key><LastModified>2016-11-15T21:32:17.000Z</LastModified><ETag>&quot;28c220e78830cc62379f3e2cb35b3881&quot;</ETag><Size>3600687</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_freebsd_amd64.tar.gz</Key><LastModified>2016-11-15T21:32:17.000Z</LastModified><ETag>&quot;def96722e2242b10f67f0217361916f0&quot;</ETag><Size>3786664</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_freebsd_arm.tar.gz</Key><LastModified>2016-11-15T21:32:17.000Z</LastModified><ETag>&quot;2f9357ae0201832fbaa512d1eecec741&quot;</ETag><Size>3502310</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_linux_386.tar.gz</Key><LastModified>2016-11-15T21:32:17.000Z</LastModified><ETag>&quot;01877da87807d41f4cb577f9734309b6&quot;</ETag><Size>3607673</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_linux_amd64.tar.gz</Key><LastModified>2016-11-15T21:32:18.000Z</LastModified><ETag>&quot;08c88b43d7053146b9b3db2bf72a1709&quot;</ETag><Size>3817512</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_linux_arm.tar.gz</Key><LastModified>2016-11-15T21:32:18.000Z</LastModified><ETag>&quot;ae82633b6e450821a4e75dc000444290&quot;</ETag><Size>3504508</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_linux_arm64.tar.gz</Key><LastModified>2016-11-15T21:32:18.000Z</LastModified><ETag>&quot;fdba4003d3018c8c058424173da2f505&quot;</ETag><Size>3533856</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_linux_ppc64.tar.gz</Key><LastModified>2016-11-15T21:32:18.000Z</LastModified><ETag>&quot;012107b282a4e33700829a15728aa0ae&quot;</ETag><Size>3545831</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_linux_ppc64le.tar.gz</Key><LastModified>2016-11-15T21:32:18.000Z</LastModified><ETag>&quot;59c28a8fa15fa82dd4d52d90d7c8ca95&quot;</ETag><Size>3502951</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/converge_0.3.0-198-g7e7c642_solaris_amd64.tar.gz</Key><LastModified>2016-11-15T21:32:19.000Z</LastModified><ETag>&quot;7e7cb0460828a5d3b848d3f6fbd08af3&quot;</ETag><Size>3762482</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-198-g7e7c642/tgz/tgz.sha256sum</Key><LastModified>2016-11-15T21:32:19.000Z</LastModified><ETag>&quot;6b7d4faab3fc8a3511283902d39aa4c2&quot;</ETag><Size>2126</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_darwin_386/converge</Key><LastModified>2016-11-02T13:51:28.000Z</LastModified><ETag>&quot;6a0fd84d17379688a4a1aa112c20f535&quot;</ETag><Size>10523532</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_darwin_386/converge.sha256sum</Key><LastModified>2016-11-02T13:51:29.000Z</LastModified><ETag>&quot;ee496eacb0ada39d6f17051d341ec8e1&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_darwin_amd64/converge</Key><LastModified>2016-11-02T13:51:29.000Z</LastModified><ETag>&quot;54280a7b42352aaa6c73f10de97ec673&quot;</ETag><Size>11804896</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-02T13:51:29.000Z</LastModified><ETag>&quot;455e8b356c18754ae88503b7e24287c4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_freebsd_386/converge</Key><LastModified>2016-11-02T13:51:29.000Z</LastModified><ETag>&quot;a90ab206079c5253103d0cb85d38f965&quot;</ETag><Size>9385248</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-02T13:51:30.000Z</LastModified><ETag>&quot;b5b2de167869aa5f2e0fc2e13cb878e7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_freebsd_amd64/converge</Key><LastModified>2016-11-02T13:51:30.000Z</LastModified><ETag>&quot;741b569c39ed7831e7d3deb76235dd1c&quot;</ETag><Size>10590752</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-02T13:51:31.000Z</LastModified><ETag>&quot;2ae39b6007e50813f50b016d4611f624&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_freebsd_arm/converge</Key><LastModified>2016-11-02T13:51:31.000Z</LastModified><ETag>&quot;b51083df3800457753f2349d3fa9e932&quot;</ETag><Size>9630152</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-02T13:51:31.000Z</LastModified><ETag>&quot;953867a83756114b249d95ec6b13b1e2&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_386/converge</Key><LastModified>2016-11-02T13:51:31.000Z</LastModified><ETag>&quot;5dd940eae652a7df8dc5ce9d1b961f20&quot;</ETag><Size>9459200</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_386/converge.sha256sum</Key><LastModified>2016-11-02T13:51:31.000Z</LastModified><ETag>&quot;b73737c8bb72b6920ac997ccba6f2024&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_amd64/converge</Key><LastModified>2016-11-02T13:51:31.000Z</LastModified><ETag>&quot;b2830775db967c34337526edbf99124b&quot;</ETag><Size>10664128</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-02T13:51:32.000Z</LastModified><ETag>&quot;e69904565c980a69dc21227ea51a525d&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_arm/converge</Key><LastModified>2016-11-02T13:51:32.000Z</LastModified><ETag>&quot;d522cf01edcdefdefc669c3681ac2c52&quot;</ETag><Size>9626448</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_arm/converge.sha256sum</Key><LastModified>2016-11-02T13:51:32.000Z</LastModified><ETag>&quot;0bc8283c19b631760c3771bdafaab939&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_arm64/converge</Key><LastModified>2016-11-02T13:51:32.000Z</LastModified><ETag>&quot;7b7493069e4c29be8e9cd78e6e5210e7&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-02T13:51:33.000Z</LastModified><ETag>&quot;7eda1ab9ee459934d2571a1e4b4e1380&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_ppc64/converge</Key><LastModified>2016-11-02T13:51:33.000Z</LastModified><ETag>&quot;76d97a7ee20e0dfcd55ce035fa454fcb&quot;</ETag><Size>11221120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-02T13:51:33.000Z</LastModified><ETag>&quot;65c3c529e7e4e8754c8adecc658e32d8&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_ppc64le/converge</Key><LastModified>2016-11-02T13:51:33.000Z</LastModified><ETag>&quot;157262ebe5bae3ae0ea60cb57662bc0b&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-02T13:51:34.000Z</LastModified><ETag>&quot;93d08047da1b7f4db22c59e646650408&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_solaris_amd64/converge</Key><LastModified>2016-11-02T13:51:34.000Z</LastModified><ETag>&quot;b9281781c4f19730aad5c3181e3c9df3&quot;</ETag><Size>10528416</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/converge_0.3.0-23-gc982126_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-02T13:51:35.000Z</LastModified><ETag>&quot;73d0fff25f11bec8d625f00d379e4fbc&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_darwin_386.tar.gz</Key><LastModified>2016-11-02T13:51:35.000Z</LastModified><ETag>&quot;19d2e75d89b2ac2c07f3599c05140edd&quot;</ETag><Size>3726883</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_darwin_amd64.tar.gz</Key><LastModified>2016-11-02T13:51:35.000Z</LastModified><ETag>&quot;c5d830ee4db7eedb83472f6dc7ee8f05&quot;</ETag><Size>3908177</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_freebsd_386.tar.gz</Key><LastModified>2016-11-02T13:51:35.000Z</LastModified><ETag>&quot;107ee3e708fe4015a7c5a2cc0974f0d5&quot;</ETag><Size>3533174</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_freebsd_amd64.tar.gz</Key><LastModified>2016-11-02T13:51:35.000Z</LastModified><ETag>&quot;025bdad89ac3f74acece24e696dac432&quot;</ETag><Size>3716429</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_freebsd_arm.tar.gz</Key><LastModified>2016-11-02T13:51:36.000Z</LastModified><ETag>&quot;3caf8ca9bc4f31f8418ae9a4d0c96692&quot;</ETag><Size>3438445</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_linux_386.tar.gz</Key><LastModified>2016-11-02T13:51:36.000Z</LastModified><ETag>&quot;9a17333a72932184734de75ddc0e3e5a&quot;</ETag><Size>3540593</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_linux_amd64.tar.gz</Key><LastModified>2016-11-02T13:51:36.000Z</LastModified><ETag>&quot;57312e75adf2267944110c1b3fb791da&quot;</ETag><Size>3746289</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_linux_arm.tar.gz</Key><LastModified>2016-11-02T13:51:37.000Z</LastModified><ETag>&quot;359ef6de755c79cf5893b5ff799f68da&quot;</ETag><Size>3438218</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_linux_arm64.tar.gz</Key><LastModified>2016-11-02T13:51:37.000Z</LastModified><ETag>&quot;878fb2c77a4a3d8c47e2271f3100a647&quot;</ETag><Size>3466717</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_linux_ppc64.tar.gz</Key><LastModified>2016-11-02T13:51:37.000Z</LastModified><ETag>&quot;ecb38dd0edf35583c86a4a6601610989&quot;</ETag><Size>3480294</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_linux_ppc64le.tar.gz</Key><LastModified>2016-11-02T13:51:37.000Z</LastModified><ETag>&quot;a7c2a41ccd116caed4aa2b96fce7440a&quot;</ETag><Size>3439548</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/converge_0.3.0-23-gc982126_solaris_amd64.tar.gz</Key><LastModified>2016-11-02T13:51:38.000Z</LastModified><ETag>&quot;6e013907beafe2b2ab2d0367ecd96b3b&quot;</ETag><Size>3690495</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-23-gc982126/tgz/tgz.sha256sum</Key><LastModified>2016-11-02T13:51:38.000Z</LastModified><ETag>&quot;8936ab204e5de07993d841578bb49913&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_darwin_386/converge</Key><LastModified>2016-11-02T14:48:25.000Z</LastModified><ETag>&quot;13633bb482e8e53b74b475af1fa6a45d&quot;</ETag><Size>10532612</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_darwin_386/converge.sha256sum</Key><LastModified>2016-11-02T14:48:25.000Z</LastModified><ETag>&quot;d166b4f65b82b815a6d5ce34550bc5a3&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_darwin_amd64/converge</Key><LastModified>2016-11-02T14:48:25.000Z</LastModified><ETag>&quot;776173db8a3cc4a04a6b7a31f09a10e1&quot;</ETag><Size>11814000</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-02T14:48:26.000Z</LastModified><ETag>&quot;d7c7d8f4aec58f763f57fd4ab544a90e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_freebsd_386/converge</Key><LastModified>2016-11-02T14:48:26.000Z</LastModified><ETag>&quot;e12a34641a66a22d57f85426b253d2be&quot;</ETag><Size>9389344</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-02T14:48:26.000Z</LastModified><ETag>&quot;814d9d1271c37eabf71c89be546741f9&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_freebsd_amd64/converge</Key><LastModified>2016-11-02T14:48:26.000Z</LastModified><ETag>&quot;7ad442b08b795e0dfc1a8b567eaa18b2&quot;</ETag><Size>10598944</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-02T14:48:27.000Z</LastModified><ETag>&quot;1e3a2bc97d2e9a61a32b0c188d764c0b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_freebsd_arm/converge</Key><LastModified>2016-11-02T14:48:27.000Z</LastModified><ETag>&quot;f4be7c51e92eaf370118b2a75116ae9a&quot;</ETag><Size>9638352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-02T14:48:29.000Z</LastModified><ETag>&quot;692796c00c7418b0bf910da1cb1fe488&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_386/converge</Key><LastModified>2016-11-02T14:48:29.000Z</LastModified><ETag>&quot;ce6144320867ff6c4e7d31990d6e3da8&quot;</ETag><Size>9463296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_386/converge.sha256sum</Key><LastModified>2016-11-02T14:48:29.000Z</LastModified><ETag>&quot;2585eafb6132140171fed3a7aefbec06&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_amd64/converge</Key><LastModified>2016-11-02T14:48:30.000Z</LastModified><ETag>&quot;fd6f70f9d9050ebd3ce78420ab7a814a&quot;</ETag><Size>10671296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-02T14:48:30.000Z</LastModified><ETag>&quot;3910bb076b387a3e6c3f5468433f0bd1&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_arm/converge</Key><LastModified>2016-11-02T14:48:30.000Z</LastModified><ETag>&quot;fabe9c969cead33fa5f4e2bb3c025257&quot;</ETag><Size>9634648</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_arm/converge.sha256sum</Key><LastModified>2016-11-02T14:48:31.000Z</LastModified><ETag>&quot;4812ff5ef9dcd291ce62d548320ac1c6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_arm64/converge</Key><LastModified>2016-11-02T14:48:31.000Z</LastModified><ETag>&quot;46d3c4a7a7e4f7b66b30d0bea9e97fbf&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-02T14:48:31.000Z</LastModified><ETag>&quot;7ba5327b0773b824fee00bf8bdd8f491&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_ppc64/converge</Key><LastModified>2016-11-02T14:48:31.000Z</LastModified><ETag>&quot;85d2c6c7fbdf37f52aed28eea571a387&quot;</ETag><Size>11221120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-02T14:48:32.000Z</LastModified><ETag>&quot;c07b8aaf6e8cf68271f1338fc4dc2023&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_ppc64le/converge</Key><LastModified>2016-11-02T14:48:32.000Z</LastModified><ETag>&quot;385cbe55009ad160e4e2b76145af3587&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-02T14:48:33.000Z</LastModified><ETag>&quot;791d254aed47cd80b7389dfe8a0aaf0c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_solaris_amd64/converge</Key><LastModified>2016-11-02T14:48:33.000Z</LastModified><ETag>&quot;9bf6cedd41beb675b41393999707531f&quot;</ETag><Size>10532512</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/converge_0.3.0-26-ga129cc1_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-02T14:48:34.000Z</LastModified><ETag>&quot;021a28e06134f5efbe240be3d22e41ba&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_darwin_386.tar.gz</Key><LastModified>2016-11-02T14:48:34.000Z</LastModified><ETag>&quot;037da6cf6d7d45d4bd95418351f715d5&quot;</ETag><Size>3728693</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_darwin_amd64.tar.gz</Key><LastModified>2016-11-02T14:48:34.000Z</LastModified><ETag>&quot;284fecdf92a195465313a71499ebfe66&quot;</ETag><Size>3911304</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_freebsd_386.tar.gz</Key><LastModified>2016-11-02T14:48:34.000Z</LastModified><ETag>&quot;5dd7af100365bbb2000fb0945eaab4d8&quot;</ETag><Size>3535188</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_freebsd_amd64.tar.gz</Key><LastModified>2016-11-02T14:48:35.000Z</LastModified><ETag>&quot;4f2762e9f1e97e860e562252d0296832&quot;</ETag><Size>3718567</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_freebsd_arm.tar.gz</Key><LastModified>2016-11-02T14:48:35.000Z</LastModified><ETag>&quot;be3459a7818bac96cabaf845ed9fffe8&quot;</ETag><Size>3437943</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_linux_386.tar.gz</Key><LastModified>2016-11-02T14:48:35.000Z</LastModified><ETag>&quot;514872f1a36973d10413a358f1416614&quot;</ETag><Size>3542247</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_linux_amd64.tar.gz</Key><LastModified>2016-11-02T14:48:35.000Z</LastModified><ETag>&quot;5dbcafc9bb7718685ec808c9a59d1a00&quot;</ETag><Size>3746828</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_linux_arm.tar.gz</Key><LastModified>2016-11-02T14:48:36.000Z</LastModified><ETag>&quot;127aa1cf849312dc66f306e998191d23&quot;</ETag><Size>3439218</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_linux_arm64.tar.gz</Key><LastModified>2016-11-02T14:48:36.000Z</LastModified><ETag>&quot;b2b7f038375ef0715ff5c3279bad9653&quot;</ETag><Size>3467616</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_linux_ppc64.tar.gz</Key><LastModified>2016-11-02T14:48:37.000Z</LastModified><ETag>&quot;8c858682c3a483e98ae956172a95db18&quot;</ETag><Size>3486431</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_linux_ppc64le.tar.gz</Key><LastModified>2016-11-02T14:48:37.000Z</LastModified><ETag>&quot;9b596054bb37c0f920588d6e40129240&quot;</ETag><Size>3441203</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/converge_0.3.0-26-ga129cc1_solaris_amd64.tar.gz</Key><LastModified>2016-11-02T14:48:37.000Z</LastModified><ETag>&quot;d56dfd04fa2b59a21b4b3de85eeb64f5&quot;</ETag><Size>3692627</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-26-ga129cc1/tgz/tgz.sha256sum</Key><LastModified>2016-11-02T14:48:38.000Z</LastModified><ETag>&quot;c8397b6c22d0b5930c7d901909719a70&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_darwin_386/converge</Key><LastModified>2016-11-02T19:34:10.000Z</LastModified><ETag>&quot;84b56034e75d8bb944eb377e40906007&quot;</ETag><Size>10537596</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_darwin_386/converge.sha256sum</Key><LastModified>2016-11-02T19:34:10.000Z</LastModified><ETag>&quot;c89bf1d48efac1ebd1edf6ac26c8a2e1&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_darwin_amd64/converge</Key><LastModified>2016-11-02T19:34:10.000Z</LastModified><ETag>&quot;4c9f50f72795b9ed231ecec29a037aaa&quot;</ETag><Size>11819024</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-02T19:34:11.000Z</LastModified><ETag>&quot;85aa016c252e7d979e47807b6e670e17&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_freebsd_386/converge</Key><LastModified>2016-11-02T19:34:11.000Z</LastModified><ETag>&quot;0a85a751676bd77935263e5c62657562&quot;</ETag><Size>9397536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-02T19:34:12.000Z</LastModified><ETag>&quot;301fe3195d462d3129ad0ebaef3f1fac&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_freebsd_amd64/converge</Key><LastModified>2016-11-02T19:34:12.000Z</LastModified><ETag>&quot;a4eeac884762a3e50247ee494b4a9365&quot;</ETag><Size>10607136</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-02T19:34:13.000Z</LastModified><ETag>&quot;34992937920176180d72d393ce5ff107&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_freebsd_arm/converge</Key><LastModified>2016-11-02T19:34:13.000Z</LastModified><ETag>&quot;0431eb7c13fd667ca2717af9f2425d83&quot;</ETag><Size>9642448</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-02T19:34:13.000Z</LastModified><ETag>&quot;79e8be82d9d4d887212b6a31a0364c49&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_386/converge</Key><LastModified>2016-11-02T19:34:13.000Z</LastModified><ETag>&quot;6fe66a6a0119df867cbab1b501f2496c&quot;</ETag><Size>9471488</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_386/converge.sha256sum</Key><LastModified>2016-11-02T19:34:14.000Z</LastModified><ETag>&quot;fc500baea8f8d71d54ab0912deb48ec4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_amd64/converge</Key><LastModified>2016-11-02T19:34:14.000Z</LastModified><ETag>&quot;e8a6fd2f0d17c0ae712a807c56309c75&quot;</ETag><Size>10676448</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-02T19:34:15.000Z</LastModified><ETag>&quot;a0da51d50493910bbdbea5c7c6662e06&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_arm/converge</Key><LastModified>2016-11-02T19:34:15.000Z</LastModified><ETag>&quot;21ff2e17dd737039062b8b47741c7d36&quot;</ETag><Size>9634648</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_arm/converge.sha256sum</Key><LastModified>2016-11-02T19:34:16.000Z</LastModified><ETag>&quot;b0121bf444ae6aafee5e7e36a5416633&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_arm64/converge</Key><LastModified>2016-11-02T19:34:16.000Z</LastModified><ETag>&quot;59f7305614baac2843e7ee376ea91169&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-02T19:34:17.000Z</LastModified><ETag>&quot;a8d385d306dc9cd36bfcd7b8ae83de82&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_ppc64/converge</Key><LastModified>2016-11-02T19:34:17.000Z</LastModified><ETag>&quot;0f5f456825cc1c50b587cc54b3804644&quot;</ETag><Size>11221120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-02T19:34:18.000Z</LastModified><ETag>&quot;6620c723e3f9c73103812c7d4228dfc6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_ppc64le/converge</Key><LastModified>2016-11-02T19:34:18.000Z</LastModified><ETag>&quot;e2870c4e31447c7546d961e69daed6ac&quot;</ETag><Size>11221120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-02T19:34:19.000Z</LastModified><ETag>&quot;dd4f61e1a13ba1dd34aa9b8dad85ce70&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_solaris_amd64/converge</Key><LastModified>2016-11-02T19:34:19.000Z</LastModified><ETag>&quot;ad7be8cbf03e12180fd3a73733d1b63c&quot;</ETag><Size>10540704</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/converge_0.3.0-29-g4b02c18_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-02T19:34:20.000Z</LastModified><ETag>&quot;d8acd4fb709119d32a966cf277eb4e81&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_darwin_386.tar.gz</Key><LastModified>2016-11-02T19:34:20.000Z</LastModified><ETag>&quot;c89eafa73d7d6a7b600a68f0fc51125f&quot;</ETag><Size>3731153</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_darwin_amd64.tar.gz</Key><LastModified>2016-11-02T19:34:22.000Z</LastModified><ETag>&quot;15c1cc8c1d5e5655882373bc9a664fb2&quot;</ETag><Size>3912682</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_freebsd_386.tar.gz</Key><LastModified>2016-11-02T19:34:24.000Z</LastModified><ETag>&quot;538b9a314c486ad30e4072f232bc07c2&quot;</ETag><Size>3536374</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_freebsd_amd64.tar.gz</Key><LastModified>2016-11-02T19:34:24.000Z</LastModified><ETag>&quot;4d51b0cf4fb2740c8a9bc4fabcd23ca2&quot;</ETag><Size>3719917</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_freebsd_arm.tar.gz</Key><LastModified>2016-11-02T19:34:24.000Z</LastModified><ETag>&quot;a692a60e82cafd476488a234139d090e&quot;</ETag><Size>3437970</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_linux_386.tar.gz</Key><LastModified>2016-11-02T19:34:25.000Z</LastModified><ETag>&quot;6240de34482b64d4639b40c32a580387&quot;</ETag><Size>3540601</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_linux_amd64.tar.gz</Key><LastModified>2016-11-02T19:34:25.000Z</LastModified><ETag>&quot;fd9cec29668c5070c3d0ba45f15ca792&quot;</ETag><Size>3748675</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_linux_arm.tar.gz</Key><LastModified>2016-11-02T19:34:25.000Z</LastModified><ETag>&quot;2e6252d6eb7da17c5229f553214f6d34&quot;</ETag><Size>3440023</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_linux_arm64.tar.gz</Key><LastModified>2016-11-02T19:34:25.000Z</LastModified><ETag>&quot;bea51b8d11dff1749b756a9f4a486cf5&quot;</ETag><Size>3469440</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_linux_ppc64.tar.gz</Key><LastModified>2016-11-02T19:34:26.000Z</LastModified><ETag>&quot;4e6a2c07f56d0710ae30bc33801db057&quot;</ETag><Size>3487009</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_linux_ppc64le.tar.gz</Key><LastModified>2016-11-02T19:34:26.000Z</LastModified><ETag>&quot;57708b19e90db53134be1ceb8a822e7b&quot;</ETag><Size>3443015</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/converge_0.3.0-29-g4b02c18_solaris_amd64.tar.gz</Key><LastModified>2016-11-02T19:34:26.000Z</LastModified><ETag>&quot;042e1a54c1aacfc82716adf6afda79fd&quot;</ETag><Size>3694813</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-29-g4b02c18/tgz/tgz.sha256sum</Key><LastModified>2016-11-02T19:34:27.000Z</LastModified><ETag>&quot;3f4bce62b253ce9eda4fdb79cfaf23d0&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_darwin_386/converge</Key><LastModified>2016-11-02T15:24:07.000Z</LastModified><ETag>&quot;393a355bc9ee83203cdfdefdfe4cc6ab&quot;</ETag><Size>10532612</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_darwin_386/converge.sha256sum</Key><LastModified>2016-11-02T15:24:08.000Z</LastModified><ETag>&quot;b7da3a42223243b53393fd309cfcc15e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_darwin_amd64/converge</Key><LastModified>2016-11-02T15:24:08.000Z</LastModified><ETag>&quot;72b1806a45e9dd468e97cebbb29b5a64&quot;</ETag><Size>11814000</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-02T15:24:09.000Z</LastModified><ETag>&quot;2203a5bb992b7c08aaa24af954972e08&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_freebsd_386/converge</Key><LastModified>2016-11-02T15:24:09.000Z</LastModified><ETag>&quot;b5fbeaaf2a5613812be57377ab4cc05f&quot;</ETag><Size>9389344</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-02T15:24:09.000Z</LastModified><ETag>&quot;389ae641f0d2cd107315439322332545&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_freebsd_amd64/converge</Key><LastModified>2016-11-02T15:24:09.000Z</LastModified><ETag>&quot;cdb741c8c35ae156e07e597bc8013a7c&quot;</ETag><Size>10598944</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-02T15:24:10.000Z</LastModified><ETag>&quot;b4e35ed24cf106019bb7c6ae64b744f7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_freebsd_arm/converge</Key><LastModified>2016-11-02T15:24:10.000Z</LastModified><ETag>&quot;91b980c07fae3043c572e837a47c01c3&quot;</ETag><Size>9638352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-02T15:24:13.000Z</LastModified><ETag>&quot;ea0892f6c933118e3aca2a7400e70b0e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_386/converge</Key><LastModified>2016-11-02T15:24:13.000Z</LastModified><ETag>&quot;55b639a940bc8f697f119910a4679403&quot;</ETag><Size>9463296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_386/converge.sha256sum</Key><LastModified>2016-11-02T15:24:13.000Z</LastModified><ETag>&quot;5a07bc9d291ba968f6c71a7e9305083b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_amd64/converge</Key><LastModified>2016-11-02T15:24:13.000Z</LastModified><ETag>&quot;a77ced420ddd5d4f2fc172d800cb1a8e&quot;</ETag><Size>10671296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-02T15:24:13.000Z</LastModified><ETag>&quot;e9994ad101527d4cca1ae55829bb4eb1&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_arm/converge</Key><LastModified>2016-11-02T15:24:13.000Z</LastModified><ETag>&quot;160ac64cd64fb9aceaf5383348339056&quot;</ETag><Size>9634648</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_arm/converge.sha256sum</Key><LastModified>2016-11-02T15:24:14.000Z</LastModified><ETag>&quot;f6fed0d50cd023a256160ed63f6880f2&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_arm64/converge</Key><LastModified>2016-11-02T15:24:14.000Z</LastModified><ETag>&quot;908a8931e2674748000a4514e623ebaa&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-02T15:24:14.000Z</LastModified><ETag>&quot;3146b93de69daf2aae21d589921efdd8&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_ppc64/converge</Key><LastModified>2016-11-02T15:24:14.000Z</LastModified><ETag>&quot;f02b2867fc2e5ce6daa7ed173add0557&quot;</ETag><Size>11221120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-02T15:24:15.000Z</LastModified><ETag>&quot;9ec1b7a8d8bc27f07bdb365aab2ded9f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_ppc64le/converge</Key><LastModified>2016-11-02T15:24:15.000Z</LastModified><ETag>&quot;3e510dad06ccccd2daf7150e6b75d11a&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-02T15:24:15.000Z</LastModified><ETag>&quot;23c2e5d1f1bb1f588e4d6c044b0d7c0f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_solaris_amd64/converge</Key><LastModified>2016-11-02T15:24:15.000Z</LastModified><ETag>&quot;733dee0505170d3e87e4f4514dc5304b&quot;</ETag><Size>10532512</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/converge_0.3.0-30-g36aa7bc_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-02T15:24:16.000Z</LastModified><ETag>&quot;b474c8cf5a0c74cd1bcf3d6efba0a79f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_darwin_386.tar.gz</Key><LastModified>2016-11-02T15:24:16.000Z</LastModified><ETag>&quot;1aa4670af7e08fead2c4e845ff804623&quot;</ETag><Size>3728694</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_darwin_amd64.tar.gz</Key><LastModified>2016-11-02T15:24:16.000Z</LastModified><ETag>&quot;c21ed8fb314c7a3d8379bc010663bd75&quot;</ETag><Size>3911303</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_freebsd_386.tar.gz</Key><LastModified>2016-11-02T15:24:16.000Z</LastModified><ETag>&quot;0456fd45688f1f0032fb7e7b405b48d9&quot;</ETag><Size>3535187</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_freebsd_amd64.tar.gz</Key><LastModified>2016-11-02T15:24:17.000Z</LastModified><ETag>&quot;cb6d45145e556d417b40287af6678099&quot;</ETag><Size>3718571</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_freebsd_arm.tar.gz</Key><LastModified>2016-11-02T15:24:17.000Z</LastModified><ETag>&quot;0af6114a987159031d333c4a5db74a10&quot;</ETag><Size>3437934</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_linux_386.tar.gz</Key><LastModified>2016-11-02T15:24:17.000Z</LastModified><ETag>&quot;f1c3e1bddf0b751cb269be088b265a0e&quot;</ETag><Size>3542215</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_linux_amd64.tar.gz</Key><LastModified>2016-11-02T15:24:18.000Z</LastModified><ETag>&quot;3f7ce081fe58e57abde429eacf9cae2a&quot;</ETag><Size>3746792</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_linux_arm.tar.gz</Key><LastModified>2016-11-02T15:24:18.000Z</LastModified><ETag>&quot;18bde4c9c2ee57a6eb842378c8b3273e&quot;</ETag><Size>3439203</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_linux_arm64.tar.gz</Key><LastModified>2016-11-02T15:24:19.000Z</LastModified><ETag>&quot;561434338bcec74014c10d033d74be1e&quot;</ETag><Size>3467608</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_linux_ppc64.tar.gz</Key><LastModified>2016-11-02T15:24:19.000Z</LastModified><ETag>&quot;9e252cbeafb65ba65865277dd5ffea03&quot;</ETag><Size>3486417</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_linux_ppc64le.tar.gz</Key><LastModified>2016-11-02T15:24:19.000Z</LastModified><ETag>&quot;39b8207ce6d73c2f43a06b7682770254&quot;</ETag><Size>3441180</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/converge_0.3.0-30-g36aa7bc_solaris_amd64.tar.gz</Key><LastModified>2016-11-02T15:24:19.000Z</LastModified><ETag>&quot;0fd4d498b25058e2d1d3557c8dc8f9dc&quot;</ETag><Size>3692623</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-30-g36aa7bc/tgz/tgz.sha256sum</Key><LastModified>2016-11-02T15:24:19.000Z</LastModified><ETag>&quot;6df64f8ce808499fb236e49d44ca0f4f&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_darwin_386/converge</Key><LastModified>2016-11-02T19:36:58.000Z</LastModified><ETag>&quot;4edba263c4cff6b633f41fc746350b0b&quot;</ETag><Size>10537596</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_darwin_386/converge.sha256sum</Key><LastModified>2016-11-02T19:36:59.000Z</LastModified><ETag>&quot;edbf2f19a6f0727ff07b442f586f51c6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_darwin_amd64/converge</Key><LastModified>2016-11-02T19:36:59.000Z</LastModified><ETag>&quot;e2ea248274f4bd890d9eae16c6f2e721&quot;</ETag><Size>11819024</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-02T19:37:00.000Z</LastModified><ETag>&quot;6578d966e02158f366362ab2b992943f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_freebsd_386/converge</Key><LastModified>2016-11-02T19:37:00.000Z</LastModified><ETag>&quot;a751f801bf78cf3c6ccf90f9bf4ad99e&quot;</ETag><Size>9397536</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-02T19:37:00.000Z</LastModified><ETag>&quot;77cab2e95fa429647f34d1224d952469&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_freebsd_amd64/converge</Key><LastModified>2016-11-02T19:37:01.000Z</LastModified><ETag>&quot;a32f906aa2e4b2964097c109fda32edc&quot;</ETag><Size>10607136</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-02T19:37:01.000Z</LastModified><ETag>&quot;81edd896612ed70604fd8456e14b2d71&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_freebsd_arm/converge</Key><LastModified>2016-11-02T19:37:01.000Z</LastModified><ETag>&quot;06d59a72ea3c98290d74d9a637681c2b&quot;</ETag><Size>9642448</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-02T19:37:02.000Z</LastModified><ETag>&quot;ac3bd0d14c918438e2ff16f0435e2a14&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_386/converge</Key><LastModified>2016-11-02T19:37:02.000Z</LastModified><ETag>&quot;98d5a2b9cc032e07a96c6f7b0fe0f59b&quot;</ETag><Size>9471488</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_386/converge.sha256sum</Key><LastModified>2016-11-02T19:37:03.000Z</LastModified><ETag>&quot;05df95fcb1b90cac66ada59127c3eb8b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_amd64/converge</Key><LastModified>2016-11-02T19:37:03.000Z</LastModified><ETag>&quot;be70ac4020768acacc96f793c5c0c1e3&quot;</ETag><Size>10676448</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-02T19:37:03.000Z</LastModified><ETag>&quot;fe22c0b817b8a1dca8a761f058c9ca17&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_arm/converge</Key><LastModified>2016-11-02T19:37:04.000Z</LastModified><ETag>&quot;06fd353c4c1d356a32b5029f0cd681d0&quot;</ETag><Size>9634648</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_arm/converge.sha256sum</Key><LastModified>2016-11-02T19:37:04.000Z</LastModified><ETag>&quot;2f009b6a3431cbcae25fabb54ef49405&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_arm64/converge</Key><LastModified>2016-11-02T19:37:04.000Z</LastModified><ETag>&quot;c79732b6b41d49023d7f302a6db4eaf3&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-02T19:37:04.000Z</LastModified><ETag>&quot;dc584903044bf11b3d72c8f8846fc782&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_ppc64/converge</Key><LastModified>2016-11-02T19:37:04.000Z</LastModified><ETag>&quot;da7df06a87c824befb6eee1ce55ef089&quot;</ETag><Size>11221120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-02T19:37:05.000Z</LastModified><ETag>&quot;1be07441dc773e421eb9caf4bbd32e00&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_ppc64le/converge</Key><LastModified>2016-11-02T19:37:05.000Z</LastModified><ETag>&quot;3a6d5a7067ad66b06035686832b4edc1&quot;</ETag><Size>11221120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-02T19:37:05.000Z</LastModified><ETag>&quot;819393c5b8151bf7833eda72b9e4d903&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_solaris_amd64/converge</Key><LastModified>2016-11-02T19:37:06.000Z</LastModified><ETag>&quot;ca6fb3965f2d048725c0cc3684938db9&quot;</ETag><Size>10540704</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/converge_0.3.0-34-gb72d341_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-02T19:37:06.000Z</LastModified><ETag>&quot;a1f25e10b7f5cfac8c9f4b78474961eb&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_darwin_386.tar.gz</Key><LastModified>2016-11-02T19:37:06.000Z</LastModified><ETag>&quot;b4702bec403858541b3205c776d5d5c4&quot;</ETag><Size>3731135</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_darwin_amd64.tar.gz</Key><LastModified>2016-11-02T19:37:07.000Z</LastModified><ETag>&quot;f57c546dd2a1e9688aa19c31283cf73e&quot;</ETag><Size>3912679</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_freebsd_386.tar.gz</Key><LastModified>2016-11-02T19:37:07.000Z</LastModified><ETag>&quot;aad351140731e144393bc8c7a0ea17e9&quot;</ETag><Size>3536376</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_freebsd_amd64.tar.gz</Key><LastModified>2016-11-02T19:37:07.000Z</LastModified><ETag>&quot;27e02931cecba2573eb749d65361fdab&quot;</ETag><Size>3719913</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_freebsd_arm.tar.gz</Key><LastModified>2016-11-02T19:37:08.000Z</LastModified><ETag>&quot;c10855f933fe4ab1eca2fed49472a4fd&quot;</ETag><Size>3437906</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_linux_386.tar.gz</Key><LastModified>2016-11-02T19:37:08.000Z</LastModified><ETag>&quot;5d5b7e43811381c25d289ac14ea32f56&quot;</ETag><Size>3540631</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_linux_amd64.tar.gz</Key><LastModified>2016-11-02T19:37:08.000Z</LastModified><ETag>&quot;eb960ebda6720bb806e48b4185a6616f&quot;</ETag><Size>3748673</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_linux_arm.tar.gz</Key><LastModified>2016-11-02T19:37:08.000Z</LastModified><ETag>&quot;d26d9c7189280f679ad9bc6372689f5e&quot;</ETag><Size>3440040</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_linux_arm64.tar.gz</Key><LastModified>2016-11-02T19:37:09.000Z</LastModified><ETag>&quot;f9debfd89ebffe658b915c2406d08e8f&quot;</ETag><Size>3469444</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_linux_ppc64.tar.gz</Key><LastModified>2016-11-02T19:37:09.000Z</LastModified><ETag>&quot;4350f04c2393e174c19b9fe267d261e7&quot;</ETag><Size>3487009</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_linux_ppc64le.tar.gz</Key><LastModified>2016-11-02T19:37:09.000Z</LastModified><ETag>&quot;8c36b6636ac6d7b2b764328c585930a9&quot;</ETag><Size>3443017</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/converge_0.3.0-34-gb72d341_solaris_amd64.tar.gz</Key><LastModified>2016-11-02T19:37:09.000Z</LastModified><ETag>&quot;4f57146d6fea640f3772383fc96ee1b5&quot;</ETag><Size>3694754</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-34-gb72d341/tgz/tgz.sha256sum</Key><LastModified>2016-11-02T19:37:09.000Z</LastModified><ETag>&quot;9f902796a24a1a2572dfd0ea32470d09&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_darwin_386/converge</Key><LastModified>2016-11-07T17:21:33.000Z</LastModified><ETag>&quot;17485ceaf626187931ab9c7a016d5911&quot;</ETag><Size>10542588</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_darwin_386/converge.sha256sum</Key><LastModified>2016-11-07T17:21:33.000Z</LastModified><ETag>&quot;b728073674bafa277055a48d6ed7c5e8&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_darwin_amd64/converge</Key><LastModified>2016-11-07T17:21:33.000Z</LastModified><ETag>&quot;983c24ce7708c11730797f3670ff1d18&quot;</ETag><Size>11828144</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:21:34.000Z</LastModified><ETag>&quot;eaec23378cb04e8469ca82a0051c432a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_freebsd_386/converge</Key><LastModified>2016-11-07T17:21:34.000Z</LastModified><ETag>&quot;f4b3d930db99b7823cba8a30b6dd0d66&quot;</ETag><Size>9401664</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-07T17:21:34.000Z</LastModified><ETag>&quot;88e2a91e5f3a1826c028ff282ac582b6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_freebsd_amd64/converge</Key><LastModified>2016-11-07T17:21:34.000Z</LastModified><ETag>&quot;e6b24af61a63b5b1b998a495294063e6&quot;</ETag><Size>10611296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:21:35.000Z</LastModified><ETag>&quot;d3311005717d1ef812c39eb598f96644&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_freebsd_arm/converge</Key><LastModified>2016-11-07T17:21:35.000Z</LastModified><ETag>&quot;22bd8ee5fdc73055cee6076330c9c801&quot;</ETag><Size>9646576</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-07T17:21:36.000Z</LastModified><ETag>&quot;9b538cdc7d01f4d64383976308b50667&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_386/converge</Key><LastModified>2016-11-07T17:21:36.000Z</LastModified><ETag>&quot;046ec09d98faa5a11f7e6b0ec72e7c1b&quot;</ETag><Size>9479712</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_386/converge.sha256sum</Key><LastModified>2016-11-07T17:21:36.000Z</LastModified><ETag>&quot;92f3a83d3e5c9d836771a1f5e8cc0dc5&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_amd64/converge</Key><LastModified>2016-11-07T17:21:36.000Z</LastModified><ETag>&quot;09e63908eb5b0708c46b28d540a6793b&quot;</ETag><Size>10683296</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:21:36.000Z</LastModified><ETag>&quot;d7795a0bd9891ea1d0e5b31458f03f9f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_arm/converge</Key><LastModified>2016-11-07T17:21:37.000Z</LastModified><ETag>&quot;0e27de66aed6f533717659d2fda8e973&quot;</ETag><Size>9642872</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_arm/converge.sha256sum</Key><LastModified>2016-11-07T17:21:37.000Z</LastModified><ETag>&quot;c4c509bc63f338a3bb90ef2d30ef8986&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_arm64/converge</Key><LastModified>2016-11-07T17:21:37.000Z</LastModified><ETag>&quot;3b80873449f48bd043b386d6853c1edb&quot;</ETag><Size>10893440</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-07T17:21:38.000Z</LastModified><ETag>&quot;80b149f93f9d93984de8eacaa6e1e93a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_ppc64/converge</Key><LastModified>2016-11-07T17:21:38.000Z</LastModified><ETag>&quot;152a56e7e950ee02f6950efa436ec35d&quot;</ETag><Size>11221184</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-07T17:21:38.000Z</LastModified><ETag>&quot;6a4f42069ceb1a2a2d3726d8eebebbfb&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_ppc64le/converge</Key><LastModified>2016-11-07T17:21:38.000Z</LastModified><ETag>&quot;95230e65fc6405dd3c0dd56feec1037e&quot;</ETag><Size>11221184</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-07T17:21:39.000Z</LastModified><ETag>&quot;8f6c70453613455a27050dbc24539f99&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_solaris_amd64/converge</Key><LastModified>2016-11-07T17:21:39.000Z</LastModified><ETag>&quot;507f4f0d44abb43d2afe681066fdcbee&quot;</ETag><Size>10544864</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/converge_0.3.0-52-ga68da17_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:21:39.000Z</LastModified><ETag>&quot;62779805def26220cb63f85f5931af7b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_darwin_386.tar.gz</Key><LastModified>2016-11-07T17:21:39.000Z</LastModified><ETag>&quot;240764db2cacad24920e68cabcf4a1cb&quot;</ETag><Size>3732954</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_darwin_amd64.tar.gz</Key><LastModified>2016-11-07T17:21:40.000Z</LastModified><ETag>&quot;165c070d67805ecc1b5d34601ec0d178&quot;</ETag><Size>3915028</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_freebsd_386.tar.gz</Key><LastModified>2016-11-07T17:21:40.000Z</LastModified><ETag>&quot;0e2c7083d3ebadc7031028d330077e71&quot;</ETag><Size>3540453</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_freebsd_amd64.tar.gz</Key><LastModified>2016-11-07T17:21:40.000Z</LastModified><ETag>&quot;cccad95f39a8b7a9df3ded8a612be0a4&quot;</ETag><Size>3723433</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_freebsd_arm.tar.gz</Key><LastModified>2016-11-07T17:21:40.000Z</LastModified><ETag>&quot;cab90b57451431417286334b488567d5&quot;</ETag><Size>3441265</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_linux_386.tar.gz</Key><LastModified>2016-11-07T17:21:40.000Z</LastModified><ETag>&quot;369164b5d0b2f0277154ceb01ffbeffe&quot;</ETag><Size>3542998</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_linux_amd64.tar.gz</Key><LastModified>2016-11-07T17:21:41.000Z</LastModified><ETag>&quot;1e1978415977a399106267fb8793f8df&quot;</ETag><Size>3751790</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_linux_arm.tar.gz</Key><LastModified>2016-11-07T17:21:41.000Z</LastModified><ETag>&quot;5a479bac27119176c4ab6b893415f05b&quot;</ETag><Size>3442408</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_linux_arm64.tar.gz</Key><LastModified>2016-11-07T17:21:41.000Z</LastModified><ETag>&quot;779ff4400b4dbb2103d6dbfd78325134&quot;</ETag><Size>3469451</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_linux_ppc64.tar.gz</Key><LastModified>2016-11-07T17:21:41.000Z</LastModified><ETag>&quot;f6566d93b54b4777f998f6d754267329&quot;</ETag><Size>3485995</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_linux_ppc64le.tar.gz</Key><LastModified>2016-11-07T17:21:42.000Z</LastModified><ETag>&quot;964fd0a234ae78f29f3edc68cc284eec&quot;</ETag><Size>3445219</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/converge_0.3.0-52-ga68da17_solaris_amd64.tar.gz</Key><LastModified>2016-11-07T17:21:42.000Z</LastModified><ETag>&quot;1660b4bcb0a4609feced909d58c353f9&quot;</ETag><Size>3695596</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-52-ga68da17/tgz/tgz.sha256sum</Key><LastModified>2016-11-07T17:21:42.000Z</LastModified><ETag>&quot;d2ee587564489272dcc4bc9409b03c04&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_darwin_386/converge</Key><LastModified>2016-11-07T17:27:12.000Z</LastModified><ETag>&quot;5ace8a6d15ef35bd90f2cbad9e421048&quot;</ETag><Size>10601320</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_darwin_386/converge.sha256sum</Key><LastModified>2016-11-07T17:27:13.000Z</LastModified><ETag>&quot;77c9957b1abf73649f35d37ee451d603&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_darwin_amd64/converge</Key><LastModified>2016-11-07T17:27:13.000Z</LastModified><ETag>&quot;05b8a8bdb803167ee7727a6cf6a4b6fe&quot;</ETag><Size>11895696</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:27:14.000Z</LastModified><ETag>&quot;da2193919113952573538716021ca80a&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_freebsd_386/converge</Key><LastModified>2016-11-07T17:27:14.000Z</LastModified><ETag>&quot;ef993d87b1055163e75c863547a194b4&quot;</ETag><Size>9451040</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-07T17:27:14.000Z</LastModified><ETag>&quot;3c60b834ddf21ac585fa67ee5aeef5a4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_freebsd_amd64/converge</Key><LastModified>2016-11-07T17:27:14.000Z</LastModified><ETag>&quot;8112762d68ef3beba93eea085993d7b0&quot;</ETag><Size>10673120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:27:15.000Z</LastModified><ETag>&quot;433a5ccc4af8bcb4f8a5f28ddc1706bb&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_freebsd_arm/converge</Key><LastModified>2016-11-07T17:27:15.000Z</LastModified><ETag>&quot;4a124d27687ae78af2b3a8ae562192b9&quot;</ETag><Size>9691800</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-07T17:27:17.000Z</LastModified><ETag>&quot;076f90ad93f0fd7caeed654cf260f0f0&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_386/converge</Key><LastModified>2016-11-07T17:27:17.000Z</LastModified><ETag>&quot;540383976a3034bd884d69f377480ae5&quot;</ETag><Size>9524992</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_386/converge.sha256sum</Key><LastModified>2016-11-07T17:27:18.000Z</LastModified><ETag>&quot;288e9fa4df621d48b6b8228716a5f3ee&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_amd64/converge</Key><LastModified>2016-11-07T17:27:18.000Z</LastModified><ETag>&quot;99056afc91320c3dcce8ab5fa880b85e&quot;</ETag><Size>10740064</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:27:19.000Z</LastModified><ETag>&quot;199ea0422f796fe42261bfbb5d79c010&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_arm/converge</Key><LastModified>2016-11-07T17:27:19.000Z</LastModified><ETag>&quot;d57c8fab2bde95c620e995238573ed5d&quot;</ETag><Size>9692192</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_arm/converge.sha256sum</Key><LastModified>2016-11-07T17:27:19.000Z</LastModified><ETag>&quot;5994da2d8808c9c3ba53b7668b4f0fa3&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_arm64/converge</Key><LastModified>2016-11-07T17:27:19.000Z</LastModified><ETag>&quot;40f1cb41a62596d1e5775f86b7d149a3&quot;</ETag><Size>10959392</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-07T17:27:20.000Z</LastModified><ETag>&quot;9d91ceed3ede32b54e98879ded113145&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_ppc64/converge</Key><LastModified>2016-11-07T17:27:20.000Z</LastModified><ETag>&quot;5770b809b8d6f2948bab8f807298a505&quot;</ETag><Size>11287136</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-07T17:27:21.000Z</LastModified><ETag>&quot;6751ee371d323926917e7b65765040b7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_ppc64le/converge</Key><LastModified>2016-11-07T17:27:21.000Z</LastModified><ETag>&quot;c9ff677797987ed43ccbb24f9c769604&quot;</ETag><Size>11287104</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-07T17:27:21.000Z</LastModified><ETag>&quot;dc8723834c54563086a0a2e71acf123f&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_solaris_amd64/converge</Key><LastModified>2016-11-07T17:27:21.000Z</LastModified><ETag>&quot;5709b47f5f7b4a86dd7e4c60b3157d99&quot;</ETag><Size>10602592</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/converge_0.3.0-61-g30faed9_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:27:22.000Z</LastModified><ETag>&quot;399c124b7cce9116a33b72b9b0592584&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_darwin_386.tar.gz</Key><LastModified>2016-11-07T17:27:22.000Z</LastModified><ETag>&quot;865520eda5bd845a0569c18443769c75&quot;</ETag><Size>3748087</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_darwin_amd64.tar.gz</Key><LastModified>2016-11-07T17:27:24.000Z</LastModified><ETag>&quot;28e8e1019b55ab8951b0c6c4f0192b19&quot;</ETag><Size>3929778</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_freebsd_386.tar.gz</Key><LastModified>2016-11-07T17:27:25.000Z</LastModified><ETag>&quot;d7774c6d4d79401eb841d925ffca2c5c&quot;</ETag><Size>3550908</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_freebsd_amd64.tar.gz</Key><LastModified>2016-11-07T17:27:25.000Z</LastModified><ETag>&quot;9820e5b474c29cd19e60478f1a3149c4&quot;</ETag><Size>3736248</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_freebsd_arm.tar.gz</Key><LastModified>2016-11-07T17:27:25.000Z</LastModified><ETag>&quot;05c5e0f497c87a774b73d2a926386a5c&quot;</ETag><Size>3457172</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_linux_386.tar.gz</Key><LastModified>2016-11-07T17:27:25.000Z</LastModified><ETag>&quot;13133aa72db6aba978a8c50af52fa29c&quot;</ETag><Size>3558309</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_linux_amd64.tar.gz</Key><LastModified>2016-11-07T17:27:26.000Z</LastModified><ETag>&quot;ba63d5f61116a5c7fc8920def96ad891&quot;</ETag><Size>3766713</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_linux_arm.tar.gz</Key><LastModified>2016-11-07T17:27:26.000Z</LastModified><ETag>&quot;f75e47bbc9c266c26d77f1dc95b68f4c&quot;</ETag><Size>3457111</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_linux_arm64.tar.gz</Key><LastModified>2016-11-07T17:27:26.000Z</LastModified><ETag>&quot;7b931138082ceba8088448df11d25bf5&quot;</ETag><Size>3484234</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_linux_ppc64.tar.gz</Key><LastModified>2016-11-07T17:27:28.000Z</LastModified><ETag>&quot;0383ff100928ff2ca9c28cd83e9f977b&quot;</ETag><Size>3500710</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_linux_ppc64le.tar.gz</Key><LastModified>2016-11-07T17:27:28.000Z</LastModified><ETag>&quot;d9ab65c8b08ee08887c108a27c0e5e91&quot;</ETag><Size>3457326</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/converge_0.3.0-61-g30faed9_solaris_amd64.tar.gz</Key><LastModified>2016-11-07T17:27:28.000Z</LastModified><ETag>&quot;ef0165c13250113d1890018c31ee03dd&quot;</ETag><Size>3709414</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-61-g30faed9/tgz/tgz.sha256sum</Key><LastModified>2016-11-07T17:27:29.000Z</LastModified><ETag>&quot;0e9cdce81a1f0ba300dce8f8d733705c&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_darwin_386/converge</Key><LastModified>2016-11-07T17:32:25.000Z</LastModified><ETag>&quot;bb60172443770e84f36d9c16b3e064ac&quot;</ETag><Size>10601320</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_darwin_386/converge.sha256sum</Key><LastModified>2016-11-07T17:32:25.000Z</LastModified><ETag>&quot;4235e4d27d3d8525811a7764b65864d0&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_darwin_amd64/converge</Key><LastModified>2016-11-07T17:32:25.000Z</LastModified><ETag>&quot;0666d37679816ab05909b29e293e7ede&quot;</ETag><Size>11895696</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:32:29.000Z</LastModified><ETag>&quot;f2b21202a895a823fce79782feafb622&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_freebsd_386/converge</Key><LastModified>2016-11-07T17:32:29.000Z</LastModified><ETag>&quot;b74178548dd5322e3d92588f0945a312&quot;</ETag><Size>9451040</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-07T17:32:29.000Z</LastModified><ETag>&quot;503c28f5a55259a7b2c2aaccfa733d1b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_freebsd_amd64/converge</Key><LastModified>2016-11-07T17:32:30.000Z</LastModified><ETag>&quot;913172d5befa9072a233996c6a7456a0&quot;</ETag><Size>10673120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:32:31.000Z</LastModified><ETag>&quot;8c25e03166254d4cb2544d58ee868b00&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_freebsd_arm/converge</Key><LastModified>2016-11-07T17:32:31.000Z</LastModified><ETag>&quot;8df0c917b83eaeaa1e38d9af0da0fde1&quot;</ETag><Size>9691800</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-07T17:32:31.000Z</LastModified><ETag>&quot;425bb41522bb0d87737d056839475576&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_386/converge</Key><LastModified>2016-11-07T17:32:32.000Z</LastModified><ETag>&quot;2b08717e8d36b7534f8f4d24b06367ce&quot;</ETag><Size>9524992</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_386/converge.sha256sum</Key><LastModified>2016-11-07T17:32:32.000Z</LastModified><ETag>&quot;02b841208c1694b0792abfee63ebfbc4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_amd64/converge</Key><LastModified>2016-11-07T17:32:32.000Z</LastModified><ETag>&quot;8a3ff8ec9acb43aa60c76ee9f542f19b&quot;</ETag><Size>10740064</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:32:32.000Z</LastModified><ETag>&quot;6daa6bfaffd7aa900d220629d810e2d4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_arm/converge</Key><LastModified>2016-11-07T17:32:32.000Z</LastModified><ETag>&quot;b7dd0fd9e2c043d48a5ed06aa9653cd9&quot;</ETag><Size>9692192</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_arm/converge.sha256sum</Key><LastModified>2016-11-07T17:32:33.000Z</LastModified><ETag>&quot;0cd52f07e6939187f05868588d7b4d46&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_arm64/converge</Key><LastModified>2016-11-07T17:32:33.000Z</LastModified><ETag>&quot;dd4eb0bb768aa8e52569bde9ef1535c6&quot;</ETag><Size>10959392</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-07T17:32:35.000Z</LastModified><ETag>&quot;3189543edee6d8d8d8a3d336b2b6c2a6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_ppc64/converge</Key><LastModified>2016-11-07T17:32:35.000Z</LastModified><ETag>&quot;0bb74234d95e7be4ed3bbc57a9a71c9f&quot;</ETag><Size>11287136</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-07T17:32:36.000Z</LastModified><ETag>&quot;daee65e0703af95a9cdf37203adfec30&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_ppc64le/converge</Key><LastModified>2016-11-07T17:32:36.000Z</LastModified><ETag>&quot;61aab5dc5a5b6f37e8bff4126cf925a2&quot;</ETag><Size>11287104</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-07T17:32:37.000Z</LastModified><ETag>&quot;27348c7995d8b4708e55b77fb88b062c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_solaris_amd64/converge</Key><LastModified>2016-11-07T17:32:37.000Z</LastModified><ETag>&quot;9fcc6d7ae161d777cfb5a9f75d8ff563&quot;</ETag><Size>10602592</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/converge_0.3.0-65-g223e51c_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-07T17:32:40.000Z</LastModified><ETag>&quot;3002d6c0c90e91d6b7d133bac0977199&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_darwin_386.tar.gz</Key><LastModified>2016-11-07T17:32:40.000Z</LastModified><ETag>&quot;90cc3728546b20ed637e458c1252645f&quot;</ETag><Size>3748079</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_darwin_amd64.tar.gz</Key><LastModified>2016-11-07T17:32:41.000Z</LastModified><ETag>&quot;a817a2e600d0f42bd98315930a5a2af9&quot;</ETag><Size>3929775</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_freebsd_386.tar.gz</Key><LastModified>2016-11-07T17:32:41.000Z</LastModified><ETag>&quot;d592f1f0f75d3ce48a6603e8e4bda04b&quot;</ETag><Size>3550908</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_freebsd_amd64.tar.gz</Key><LastModified>2016-11-07T17:32:41.000Z</LastModified><ETag>&quot;fcc25cbc1c3294c2d83e0df83f821835&quot;</ETag><Size>3736250</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_freebsd_arm.tar.gz</Key><LastModified>2016-11-07T17:32:41.000Z</LastModified><ETag>&quot;3105f8368f67c229295ba917be988d37&quot;</ETag><Size>3457175</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_linux_386.tar.gz</Key><LastModified>2016-11-07T17:32:42.000Z</LastModified><ETag>&quot;81641ec2bc74291ddb602d6b5c83c0c5&quot;</ETag><Size>3558307</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_linux_amd64.tar.gz</Key><LastModified>2016-11-07T17:32:42.000Z</LastModified><ETag>&quot;00137a89bcb3e780a644638c03eb7873&quot;</ETag><Size>3766707</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_linux_arm.tar.gz</Key><LastModified>2016-11-07T17:32:43.000Z</LastModified><ETag>&quot;b09f745634ebf886a9b7c08cd9c9edf2&quot;</ETag><Size>3457111</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_linux_arm64.tar.gz</Key><LastModified>2016-11-07T17:32:43.000Z</LastModified><ETag>&quot;8abedb580de8e10c02a2e4c52c743ff8&quot;</ETag><Size>3484235</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_linux_ppc64.tar.gz</Key><LastModified>2016-11-07T17:32:43.000Z</LastModified><ETag>&quot;87756bac2ddf1d607eff9a73a07b945e&quot;</ETag><Size>3500706</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_linux_ppc64le.tar.gz</Key><LastModified>2016-11-07T17:32:44.000Z</LastModified><ETag>&quot;5063f3b577f7d6f4b0b5f9703cce5410&quot;</ETag><Size>3457325</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/converge_0.3.0-65-g223e51c_solaris_amd64.tar.gz</Key><LastModified>2016-11-07T17:32:44.000Z</LastModified><ETag>&quot;a9de58bf9b045d050ebaf32e356b8e01&quot;</ETag><Size>3709414</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-65-g223e51c/tgz/tgz.sha256sum</Key><LastModified>2016-11-07T17:32:44.000Z</LastModified><ETag>&quot;45ac29c16d2dd4871625c4928cd5da01&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_darwin_386/converge</Key><LastModified>2016-11-07T21:36:49.000Z</LastModified><ETag>&quot;8cdde10fe037ed24fcd47a33c224f8d2&quot;</ETag><Size>10601320</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_darwin_386/converge.sha256sum</Key><LastModified>2016-11-07T21:36:49.000Z</LastModified><ETag>&quot;572d370683f3e4365283c50b4f51694b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_darwin_amd64/converge</Key><LastModified>2016-11-07T21:36:49.000Z</LastModified><ETag>&quot;697a9669bce466873bfed469642370f8&quot;</ETag><Size>11895696</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-07T21:36:50.000Z</LastModified><ETag>&quot;c78b2863ea93ef2b846ce1f3f98b3c97&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_freebsd_386/converge</Key><LastModified>2016-11-07T21:36:50.000Z</LastModified><ETag>&quot;e6f1efa5682137fdb209a33f79170229&quot;</ETag><Size>9451040</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-07T21:36:51.000Z</LastModified><ETag>&quot;b30ef3057e59cde923c35f11a8e2fb87&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_freebsd_amd64/converge</Key><LastModified>2016-11-07T21:36:51.000Z</LastModified><ETag>&quot;04fe0aa620e2978de62de8d4a2d3bb0c&quot;</ETag><Size>10673120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-07T21:36:51.000Z</LastModified><ETag>&quot;b1d1f3566f912d8531178587d4f061ed&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_freebsd_arm/converge</Key><LastModified>2016-11-07T21:36:51.000Z</LastModified><ETag>&quot;871bcfb7877e7b5d6b2c2b05c4bbc2af&quot;</ETag><Size>9691800</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-07T21:36:52.000Z</LastModified><ETag>&quot;69191c31c1283a29cb9775625b727b78&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_386/converge</Key><LastModified>2016-11-07T21:36:52.000Z</LastModified><ETag>&quot;c3b61977c3dfef5c39cd110522cffb55&quot;</ETag><Size>9524992</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_386/converge.sha256sum</Key><LastModified>2016-11-07T21:36:53.000Z</LastModified><ETag>&quot;ef4dfbe1e76667c38bd7638cb87cd358&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_amd64/converge</Key><LastModified>2016-11-07T21:36:53.000Z</LastModified><ETag>&quot;a4344d5c9ec8143e059ac541b755ecf2&quot;</ETag><Size>10740352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-07T21:36:53.000Z</LastModified><ETag>&quot;216c03abedff5bc4cc6ad33b6dbec790&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_arm/converge</Key><LastModified>2016-11-07T21:36:53.000Z</LastModified><ETag>&quot;5fb83d818ca97823f8fbda0c1e9c7028&quot;</ETag><Size>9692192</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_arm/converge.sha256sum</Key><LastModified>2016-11-07T21:36:54.000Z</LastModified><ETag>&quot;1d6fa7cb5abf4ff34cab5072b17c3700&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_arm64/converge</Key><LastModified>2016-11-07T21:36:54.000Z</LastModified><ETag>&quot;b6ad173e80a198a833e9178efee478b1&quot;</ETag><Size>10959392</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-07T21:36:54.000Z</LastModified><ETag>&quot;cbb5d11029c0d18cfb20fa70b28efa4c&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_ppc64/converge</Key><LastModified>2016-11-07T21:36:54.000Z</LastModified><ETag>&quot;f37ce199c51ee01ac46ef0bde903bc3e&quot;</ETag><Size>11287136</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-07T21:36:55.000Z</LastModified><ETag>&quot;4b31758916468717fc98c66c7c134f67&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_ppc64le/converge</Key><LastModified>2016-11-07T21:36:55.000Z</LastModified><ETag>&quot;1fbc40c51a1cbcc99bce91bdd84bc509&quot;</ETag><Size>11287104</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-07T21:36:55.000Z</LastModified><ETag>&quot;f7ac4cab3a7cf5d4558ca71521bb2d1b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_solaris_amd64/converge</Key><LastModified>2016-11-07T21:36:55.000Z</LastModified><ETag>&quot;2749fab2844c4577d9f205b1f96d05c1&quot;</ETag><Size>10602592</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/converge_0.3.0-67-g886d6cd_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-07T21:36:56.000Z</LastModified><ETag>&quot;6fae40a1239cd5afc40b805f4b62402b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_darwin_386.tar.gz</Key><LastModified>2016-11-07T21:36:56.000Z</LastModified><ETag>&quot;be4de82b81e9a9e7f90af655f85d4b03&quot;</ETag><Size>3748078</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_darwin_amd64.tar.gz</Key><LastModified>2016-11-07T21:36:56.000Z</LastModified><ETag>&quot;7d7529050fed2aebc5326ddaa7e6ce73&quot;</ETag><Size>3929807</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_freebsd_386.tar.gz</Key><LastModified>2016-11-07T21:36:56.000Z</LastModified><ETag>&quot;0bd30dadabe47a8aea4b27b4ecaab9db&quot;</ETag><Size>3551003</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_freebsd_amd64.tar.gz</Key><LastModified>2016-11-07T21:36:57.000Z</LastModified><ETag>&quot;a1ccddf4806c4fc21985f32997975ef3&quot;</ETag><Size>3736228</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_freebsd_arm.tar.gz</Key><LastModified>2016-11-07T21:36:57.000Z</LastModified><ETag>&quot;23fa215bca3a7893b0ee2ed8f1344e20&quot;</ETag><Size>3456860</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_linux_386.tar.gz</Key><LastModified>2016-11-07T21:36:58.000Z</LastModified><ETag>&quot;20d5e3b4c6a40b814a6bcbd9b96b6fd2&quot;</ETag><Size>3558306</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_linux_amd64.tar.gz</Key><LastModified>2016-11-07T21:36:58.000Z</LastModified><ETag>&quot;45eb2a94b29e69111946e7a524444e79&quot;</ETag><Size>3766458</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_linux_arm.tar.gz</Key><LastModified>2016-11-07T21:36:59.000Z</LastModified><ETag>&quot;d73789d10d508bf6270dda3d7fd606a6&quot;</ETag><Size>3457084</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_linux_arm64.tar.gz</Key><LastModified>2016-11-07T21:36:59.000Z</LastModified><ETag>&quot;1a0805aac06df92a4c0aaf416f24195b&quot;</ETag><Size>3484303</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_linux_ppc64.tar.gz</Key><LastModified>2016-11-07T21:37:00.000Z</LastModified><ETag>&quot;738e9718ef64c919409145dffc8e4ad8&quot;</ETag><Size>3500034</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_linux_ppc64le.tar.gz</Key><LastModified>2016-11-07T21:37:01.000Z</LastModified><ETag>&quot;bd1787537fe5a3bf37097b3d3d3027d0&quot;</ETag><Size>3457256</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/converge_0.3.0-67-g886d6cd_solaris_amd64.tar.gz</Key><LastModified>2016-11-07T21:37:01.000Z</LastModified><ETag>&quot;acc0645697f3d1c05d837b3ce363af02&quot;</ETag><Size>3709211</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-67-g886d6cd/tgz/tgz.sha256sum</Key><LastModified>2016-11-07T21:37:01.000Z</LastModified><ETag>&quot;39fa22a748f803f48c8a8b85f7b4a319&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_darwin_386/converge</Key><LastModified>2016-11-08T16:36:25.000Z</LastModified><ETag>&quot;186beca10a31edc170d8d5217ff07efc&quot;</ETag><Size>10601320</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_darwin_386/converge.sha256sum</Key><LastModified>2016-11-08T16:36:25.000Z</LastModified><ETag>&quot;4f30a2ec07a8a29464790283279bd61e&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_darwin_amd64/converge</Key><LastModified>2016-11-08T16:36:26.000Z</LastModified><ETag>&quot;a30d672fd94f93e56b4a76af6e3a05d4&quot;</ETag><Size>11895696</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-08T16:36:26.000Z</LastModified><ETag>&quot;6afdfe1822a7fc98858cd0fd6dc75733&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_freebsd_386/converge</Key><LastModified>2016-11-08T16:36:26.000Z</LastModified><ETag>&quot;ee31082227fdc983cc632566ffe2041a&quot;</ETag><Size>9451040</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-08T16:36:27.000Z</LastModified><ETag>&quot;0085548db0e2184be5c10044dad767a8&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_freebsd_amd64/converge</Key><LastModified>2016-11-08T16:36:27.000Z</LastModified><ETag>&quot;362fb2e8b0f0dd7df64f26b25c7ad983&quot;</ETag><Size>10673120</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-08T16:36:27.000Z</LastModified><ETag>&quot;e2b166af43beb2f61775fb3d3e4af8b2&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_freebsd_arm/converge</Key><LastModified>2016-11-08T16:36:27.000Z</LastModified><ETag>&quot;6eaec6179f60373ec54e5d48272c4431&quot;</ETag><Size>9691800</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-08T16:36:28.000Z</LastModified><ETag>&quot;491e9f0165e0ecd39fd38a6fde49a5c4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_386/converge</Key><LastModified>2016-11-08T16:36:28.000Z</LastModified><ETag>&quot;dffc5bee6d8a623cb3080f55d4bc8555&quot;</ETag><Size>9524992</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_386/converge.sha256sum</Key><LastModified>2016-11-08T16:36:28.000Z</LastModified><ETag>&quot;70c6bcfd22f6a4bfbdcf263856e1b560&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_amd64/converge</Key><LastModified>2016-11-08T16:36:28.000Z</LastModified><ETag>&quot;985db0cb2c1c560194f77a0bdc385d03&quot;</ETag><Size>10740352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-08T16:36:29.000Z</LastModified><ETag>&quot;e25450794c766fcc0733ec3f7f846e67&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_arm/converge</Key><LastModified>2016-11-08T16:36:29.000Z</LastModified><ETag>&quot;096dcb559d38160af3235d1c4a79545b&quot;</ETag><Size>9692192</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_arm/converge.sha256sum</Key><LastModified>2016-11-08T16:36:30.000Z</LastModified><ETag>&quot;2e6860e2e9f7388317761717f875e1d9&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_arm64/converge</Key><LastModified>2016-11-08T16:36:30.000Z</LastModified><ETag>&quot;f8f5f44040ed751875194f58586d9c78&quot;</ETag><Size>10959392</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-08T16:36:30.000Z</LastModified><ETag>&quot;49066378df2fa95a6883d094a7cc8a15&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_ppc64/converge</Key><LastModified>2016-11-08T16:36:31.000Z</LastModified><ETag>&quot;545a8590c192cb1890ec5bf00934432e&quot;</ETag><Size>11287136</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-08T16:36:32.000Z</LastModified><ETag>&quot;b7cc5e6561dfbab9c6871f91620550b4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_ppc64le/converge</Key><LastModified>2016-11-08T16:36:32.000Z</LastModified><ETag>&quot;7ec2ebbdc3a136468f076fdca5c54b4a&quot;</ETag><Size>11287104</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-08T16:36:32.000Z</LastModified><ETag>&quot;c044bdc0aa297fe2af95f1d9ddacc824&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_solaris_amd64/converge</Key><LastModified>2016-11-08T16:36:32.000Z</LastModified><ETag>&quot;5f115511f33d32e927ca91b23445a10b&quot;</ETag><Size>10602592</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/converge_0.3.0-70-g6f0b1e8_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-08T16:36:33.000Z</LastModified><ETag>&quot;3909708fcffd8a0bf0c420a1408beaca&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_darwin_386.tar.gz</Key><LastModified>2016-11-08T16:36:33.000Z</LastModified><ETag>&quot;bc2a6bf34ed422665e5d2ee3cc2ac6ca&quot;</ETag><Size>3748072</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_darwin_amd64.tar.gz</Key><LastModified>2016-11-08T16:36:33.000Z</LastModified><ETag>&quot;b86f2ad59887af71c4c91292cd084bf2&quot;</ETag><Size>3929807</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_freebsd_386.tar.gz</Key><LastModified>2016-11-08T16:36:34.000Z</LastModified><ETag>&quot;4e4a12b791a3c5058bc635505e2a6190&quot;</ETag><Size>3550936</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_freebsd_amd64.tar.gz</Key><LastModified>2016-11-08T16:36:34.000Z</LastModified><ETag>&quot;732bdd4d64be056acddeb6a6c86d8c38&quot;</ETag><Size>3736230</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_freebsd_arm.tar.gz</Key><LastModified>2016-11-08T16:36:34.000Z</LastModified><ETag>&quot;a3c677152217416ee54db5a17daa0a4d&quot;</ETag><Size>3456757</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_linux_386.tar.gz</Key><LastModified>2016-11-08T16:36:34.000Z</LastModified><ETag>&quot;525851dd20ae7a945c2258f1397f7083&quot;</ETag><Size>3558154</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_linux_amd64.tar.gz</Key><LastModified>2016-11-08T16:36:35.000Z</LastModified><ETag>&quot;0474696900c1b0a67aad02a7eb95a0d1&quot;</ETag><Size>3766534</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_linux_arm.tar.gz</Key><LastModified>2016-11-08T16:36:35.000Z</LastModified><ETag>&quot;e8a543dd49a42e6947ac3588ca5596f0&quot;</ETag><Size>3457097</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_linux_arm64.tar.gz</Key><LastModified>2016-11-08T16:36:35.000Z</LastModified><ETag>&quot;aeac7a3d51168da728d1ae2fb307ca3c&quot;</ETag><Size>3484302</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_linux_ppc64.tar.gz</Key><LastModified>2016-11-08T16:36:36.000Z</LastModified><ETag>&quot;795a32330c32ac6fea68a1feb514370d&quot;</ETag><Size>3500032</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_linux_ppc64le.tar.gz</Key><LastModified>2016-11-08T16:36:36.000Z</LastModified><ETag>&quot;24ac74c6e7412e98693672018188c3fe&quot;</ETag><Size>3457255</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/converge_0.3.0-70-g6f0b1e8_solaris_amd64.tar.gz</Key><LastModified>2016-11-08T16:36:36.000Z</LastModified><ETag>&quot;f8689ac533cd7b9e6d0307dbfc8c9a20&quot;</ETag><Size>3709210</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-70-g6f0b1e8/tgz/tgz.sha256sum</Key><LastModified>2016-11-08T16:36:36.000Z</LastModified><ETag>&quot;a3cbd862215a448af7a95d7111e35da5&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_darwin_386/converge</Key><LastModified>2016-10-28T14:22:10.000Z</LastModified><ETag>&quot;199a37179268a4bf567f3a8ca914749e&quot;</ETag><Size>10518956</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_darwin_386/converge.sha256sum</Key><LastModified>2016-10-28T14:22:11.000Z</LastModified><ETag>&quot;045f1987873267f767f6a8d0cd0beb80&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_darwin_amd64/converge</Key><LastModified>2016-10-28T14:22:11.000Z</LastModified><ETag>&quot;c364edf90bd369cda2dae82b826d47e1&quot;</ETag><Size>11800288</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_darwin_amd64/converge.sha256sum</Key><LastModified>2016-10-28T14:22:13.000Z</LastModified><ETag>&quot;90bed1935a18cdfc3b351c52028f673b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_freebsd_386/converge</Key><LastModified>2016-10-28T14:22:13.000Z</LastModified><ETag>&quot;520b8d0d3ff7d5930c0996ed27886095&quot;</ETag><Size>9377056</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_freebsd_386/converge.sha256sum</Key><LastModified>2016-10-28T14:22:14.000Z</LastModified><ETag>&quot;cbb3af3f5065eb2162c60862e7324859&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_freebsd_amd64/converge</Key><LastModified>2016-10-28T14:22:14.000Z</LastModified><ETag>&quot;3d22d186fc9add468f5371044be90260&quot;</ETag><Size>10590752</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-10-28T14:22:14.000Z</LastModified><ETag>&quot;00c4f69df98f55240e3b9d128cef0a18&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_freebsd_arm/converge</Key><LastModified>2016-10-28T14:22:14.000Z</LastModified><ETag>&quot;bf69cb9d3c11433bea451c436156de2f&quot;</ETag><Size>9626056</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_freebsd_arm/converge.sha256sum</Key><LastModified>2016-10-28T14:22:15.000Z</LastModified><ETag>&quot;53f40201f6782a1424528e25c4abdb21&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_386/converge</Key><LastModified>2016-10-28T14:22:15.000Z</LastModified><ETag>&quot;ab1374b696bdfd40acdda011ba2d3872&quot;</ETag><Size>9455104</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_386/converge.sha256sum</Key><LastModified>2016-10-28T14:22:15.000Z</LastModified><ETag>&quot;638447898df7e20546ff42193bc0c5b9&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_amd64/converge</Key><LastModified>2016-10-28T14:22:15.000Z</LastModified><ETag>&quot;37892b27922a8471946aa98aa10b5b22&quot;</ETag><Size>10658944</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_amd64/converge.sha256sum</Key><LastModified>2016-10-28T14:22:16.000Z</LastModified><ETag>&quot;ebc9a23c72abf2b9d2f392d194a229a6&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_arm/converge</Key><LastModified>2016-10-28T14:22:16.000Z</LastModified><ETag>&quot;1c64cfb05d7b65e3d19a549f07f53b04&quot;</ETag><Size>9622352</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_arm/converge.sha256sum</Key><LastModified>2016-10-28T14:22:17.000Z</LastModified><ETag>&quot;f976554385b90b3b9e7f7caf1d559dd7&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_arm64/converge</Key><LastModified>2016-10-28T14:22:17.000Z</LastModified><ETag>&quot;26a46b653d1d5b0d1fe764ba60d626e0&quot;</ETag><Size>10827840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_arm64/converge.sha256sum</Key><LastModified>2016-10-28T14:22:17.000Z</LastModified><ETag>&quot;03136c3b29bc7c2145b0c0454f3517a4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_ppc64/converge</Key><LastModified>2016-10-28T14:22:17.000Z</LastModified><ETag>&quot;83db8b1238fbbdd9f88de54873bda8c1&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_ppc64/converge.sha256sum</Key><LastModified>2016-10-28T14:22:18.000Z</LastModified><ETag>&quot;6c1b53ad8cdc54da67505d7488460492&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_ppc64le/converge</Key><LastModified>2016-10-28T14:22:18.000Z</LastModified><ETag>&quot;8061d0d8a4d6dd7263a5b4795b1b409a&quot;</ETag><Size>11155584</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-10-28T14:22:18.000Z</LastModified><ETag>&quot;42df2a5178990a18c33f3757070aacbc&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_solaris_amd64/converge</Key><LastModified>2016-10-28T14:22:19.000Z</LastModified><ETag>&quot;b422348e8bfce29397894ecab77c360d&quot;</ETag><Size>10520224</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/converge_0.3.0-8-g4dda174_solaris_amd64/converge.sha256sum</Key><LastModified>2016-10-28T14:22:19.000Z</LastModified><ETag>&quot;ec1feb4e072ace203ce70c8faf53fbfb&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_darwin_386.tar.gz</Key><LastModified>2016-10-28T14:22:19.000Z</LastModified><ETag>&quot;0484110f87ae8fa8f4d69e5e7a95e370&quot;</ETag><Size>3724787</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_darwin_amd64.tar.gz</Key><LastModified>2016-10-28T14:22:20.000Z</LastModified><ETag>&quot;a9c44e699428284ed22c78e383e27405&quot;</ETag><Size>3906179</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_freebsd_386.tar.gz</Key><LastModified>2016-10-28T14:22:20.000Z</LastModified><ETag>&quot;20b0275cfd8961e7f790ab336587e4ee&quot;</ETag><Size>3531556</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_freebsd_amd64.tar.gz</Key><LastModified>2016-10-28T14:22:20.000Z</LastModified><ETag>&quot;25c787aaf41b2f4ba5efc08b2e1ce1bd&quot;</ETag><Size>3714253</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_freebsd_arm.tar.gz</Key><LastModified>2016-10-28T14:22:20.000Z</LastModified><ETag>&quot;d276ce43ca072a8422feeeaa212cd790&quot;</ETag><Size>3434747</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_linux_386.tar.gz</Key><LastModified>2016-10-28T14:22:20.000Z</LastModified><ETag>&quot;2fca49827a34254be1973055550e82e6&quot;</ETag><Size>3537263</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_linux_amd64.tar.gz</Key><LastModified>2016-10-28T14:22:21.000Z</LastModified><ETag>&quot;6f01cb5ad2315f0c523183ced5f54c80&quot;</ETag><Size>3744391</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_linux_arm.tar.gz</Key><LastModified>2016-10-28T14:22:21.000Z</LastModified><ETag>&quot;1edc28b305a12c8ad6e0cf05ef71a82e&quot;</ETag><Size>3438000</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_linux_arm64.tar.gz</Key><LastModified>2016-10-28T14:22:21.000Z</LastModified><ETag>&quot;c5e146c51baab4399a20cebec536ebe9&quot;</ETag><Size>3462169</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_linux_ppc64.tar.gz</Key><LastModified>2016-10-28T14:22:22.000Z</LastModified><ETag>&quot;72a371bd322a44a3615492c533d86d74&quot;</ETag><Size>3477770</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_linux_ppc64le.tar.gz</Key><LastModified>2016-10-28T14:22:22.000Z</LastModified><ETag>&quot;2070c51669f77cde01f7cbf32243fed7&quot;</ETag><Size>3436386</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/converge_0.3.0-8-g4dda174_solaris_amd64.tar.gz</Key><LastModified>2016-10-28T14:22:22.000Z</LastModified><ETag>&quot;33602c8f5c5e3e7f553b8f1e4b7870ca&quot;</ETag><Size>3686508</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-8-g4dda174/tgz/tgz.sha256sum</Key><LastModified>2016-10-28T14:22:22.000Z</LastModified><ETag>&quot;8de07c69ec3d354dfab0b0848028b423&quot;</ETag><Size>2102</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_darwin_386/converge</Key><LastModified>2016-11-09T16:43:27.000Z</LastModified><ETag>&quot;064e8110fe8302135bb54e4c7d1c8d66&quot;</ETag><Size>10647152</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_darwin_386/converge.sha256sum</Key><LastModified>2016-11-09T16:43:27.000Z</LastModified><ETag>&quot;0977af915053d642db606784d0c9a60d&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_darwin_amd64/converge</Key><LastModified>2016-11-09T16:43:27.000Z</LastModified><ETag>&quot;b1cf0222353d9eccf5abde37aa619a33&quot;</ETag><Size>11945840</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_darwin_amd64/converge.sha256sum</Key><LastModified>2016-11-09T16:43:28.000Z</LastModified><ETag>&quot;dd390bd2c6649be677076a65670ade62&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_freebsd_386/converge</Key><LastModified>2016-11-09T16:43:28.000Z</LastModified><ETag>&quot;8d966435164a9254b010a85d31fc7799&quot;</ETag><Size>9492256</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_freebsd_386/converge.sha256sum</Key><LastModified>2016-11-09T16:43:28.000Z</LastModified><ETag>&quot;ccb27a20987102c0db6230989d8c6030&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_freebsd_amd64/converge</Key><LastModified>2016-11-09T16:43:29.000Z</LastModified><ETag>&quot;00b0b5f1a0869fa2584fc32b14010517&quot;</ETag><Size>10714496</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_freebsd_amd64/converge.sha256sum</Key><LastModified>2016-11-09T16:43:29.000Z</LastModified><ETag>&quot;5be64cc91c09a6a54458e0edb4e26769&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_freebsd_arm/converge</Key><LastModified>2016-11-09T16:43:29.000Z</LastModified><ETag>&quot;c4000443f6b5494650f60d1fc7e2d4f6&quot;</ETag><Size>9732968</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_freebsd_arm/converge.sha256sum</Key><LastModified>2016-11-09T16:43:29.000Z</LastModified><ETag>&quot;3a49ac541794a83219dfcfe8f7b32568&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_386/converge</Key><LastModified>2016-11-09T16:43:29.000Z</LastModified><ETag>&quot;ec02abbc11418a181b00c21e17d5585b&quot;</ETag><Size>9570304</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_386/converge.sha256sum</Key><LastModified>2016-11-09T16:43:31.000Z</LastModified><ETag>&quot;9f2c7c558e81dd38f42318a4d6697895&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_amd64/converge</Key><LastModified>2016-11-09T16:43:31.000Z</LastModified><ETag>&quot;0a39ffbeb74387a37658648f1c14dc55&quot;</ETag><Size>10785312</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_amd64/converge.sha256sum</Key><LastModified>2016-11-09T16:43:32.000Z</LastModified><ETag>&quot;89c72ce7bb9b40343646fefc4b3f795b&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_arm/converge</Key><LastModified>2016-11-09T16:43:32.000Z</LastModified><ETag>&quot;9b89d0b52441b640ae9df6ea2a96da88&quot;</ETag><Size>9729264</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_arm/converge.sha256sum</Key><LastModified>2016-11-09T16:43:32.000Z</LastModified><ETag>&quot;bf1863a4020fc09acc32256da42951a4&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_arm64/converge</Key><LastModified>2016-11-09T16:43:32.000Z</LastModified><ETag>&quot;58d36dae3b701ebb589412bd12d2a7ed&quot;</ETag><Size>10959808</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_arm64/converge.sha256sum</Key><LastModified>2016-11-09T16:43:33.000Z</LastModified><ETag>&quot;aa8a81ea1094df58a304eb11818f0804&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_ppc64/converge</Key><LastModified>2016-11-09T16:43:33.000Z</LastModified><ETag>&quot;92e4f609bce6ba4d1bd0e1ab63c18a9b&quot;</ETag><Size>11287552</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_ppc64/converge.sha256sum</Key><LastModified>2016-11-09T16:43:33.000Z</LastModified><ETag>&quot;3216ae23d98febbbf2fdc42bcca55894&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_ppc64le/converge</Key><LastModified>2016-11-09T16:43:33.000Z</LastModified><ETag>&quot;1ebca0d7131582108854857183d581ba&quot;</ETag><Size>11287520</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_linux_ppc64le/converge.sha256sum</Key><LastModified>2016-11-09T16:43:34.000Z</LastModified><ETag>&quot;da009ce645eb149f6a66696390c7b6cd&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_solaris_amd64/converge</Key><LastModified>2016-11-09T16:43:34.000Z</LastModified><ETag>&quot;4ae499c872bdbf675100df73f6c4c56a&quot;</ETag><Size>10648064</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/converge_0.3.0-85-gb904201_solaris_amd64/converge.sha256sum</Key><LastModified>2016-11-09T16:43:34.000Z</LastModified><ETag>&quot;0b6c002e51c683009a4988d932c84670&quot;</ETag><Size>77</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_darwin_386.tar.gz</Key><LastModified>2016-11-09T16:43:34.000Z</LastModified><ETag>&quot;9ddd9ab0cd0a78cecf371e3107a3e7fa&quot;</ETag><Size>3762148</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_darwin_amd64.tar.gz</Key><LastModified>2016-11-09T16:43:35.000Z</LastModified><ETag>&quot;05615bc328946891c860502a7e2f85a6&quot;</ETag><Size>3946986</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_freebsd_386.tar.gz</Key><LastModified>2016-11-09T16:43:35.000Z</LastModified><ETag>&quot;9326f562fd13f0bb11d988fbe7f5baea&quot;</ETag><Size>3565825</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_freebsd_amd64.tar.gz</Key><LastModified>2016-11-09T16:43:35.000Z</LastModified><ETag>&quot;6fb0779e5e9b4760e874b15989eaddb7&quot;</ETag><Size>3751322</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_freebsd_arm.tar.gz</Key><LastModified>2016-11-09T16:43:36.000Z</LastModified><ETag>&quot;cfefb7456d21310f69bb4a73da7996a6&quot;</ETag><Size>3469492</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_linux_386.tar.gz</Key><LastModified>2016-11-09T16:43:36.000Z</LastModified><ETag>&quot;11ca822354f8cdbf217304fea248d1f0&quot;</ETag><Size>3570173</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_linux_amd64.tar.gz</Key><LastModified>2016-11-09T16:43:36.000Z</LastModified><ETag>&quot;38faa655fc916d0ed80e559034d96df2&quot;</ETag><Size>3781432</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_linux_arm.tar.gz</Key><LastModified>2016-11-09T16:43:36.000Z</LastModified><ETag>&quot;d160d4c52c704249a524049d95fe6cd5&quot;</ETag><Size>3469983</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_linux_arm64.tar.gz</Key><LastModified>2016-11-09T16:43:36.000Z</LastModified><ETag>&quot;fab6937e2f4fdb613e8d4a86ea477c83&quot;</ETag><Size>3498433</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_linux_ppc64.tar.gz</Key><LastModified>2016-11-09T16:43:37.000Z</LastModified><ETag>&quot;39d200bc7093b44d5c808ab4f5431478&quot;</ETag><Size>3512480</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_linux_ppc64le.tar.gz</Key><LastModified>2016-11-09T16:43:37.000Z</LastModified><ETag>&quot;83d1a727e834623847a7a227335e7809&quot;</ETag><Size>3472544</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/converge_0.3.0-85-gb904201_solaris_amd64.tar.gz</Key><LastModified>2016-11-09T16:43:37.000Z</LastModified><ETag>&quot;30c020888fed67925c2c89dc5bfaaf76&quot;</ETag><Size>3725971</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-85-gb904201/tgz/tgz.sha256sum</Key><LastModified>2016-11-09T16:43:37.000Z</LastModified><ETag>&quot;06f6adcbfafd88450f7667b46231aaee&quot;</ETag><Size>2114</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_darwin_386/converge</Key><LastModified>2016-10-24T16:45:46.000Z</LastModified><ETag>&quot;037dad83eb4963962ba9bf4e134121d4&quot;</ETag><Size>10492160</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_darwin_amd64/converge</Key><LastModified>2016-10-24T16:45:46.000Z</LastModified><ETag>&quot;0ae45941b4f1c3b33447e7ae617115e8&quot;</ETag><Size>11777472</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_freebsd_386/converge</Key><LastModified>2016-10-24T16:45:47.000Z</LastModified><ETag>&quot;1033f691c50bb90099ce8d22bd43f854&quot;</ETag><Size>9352416</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_freebsd_amd64/converge</Key><LastModified>2016-10-24T16:45:47.000Z</LastModified><ETag>&quot;9e33beac26a8ae225b2efd2004a1b5fc&quot;</ETag><Size>10566048</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_freebsd_arm/converge</Key><LastModified>2016-10-24T16:45:48.000Z</LastModified><ETag>&quot;ee095c37a15e15c7c949b2e71f212e98&quot;</ETag><Size>9601432</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_386/converge</Key><LastModified>2016-10-24T16:45:48.000Z</LastModified><ETag>&quot;30fd13cd16438a315025fa420f2d491e&quot;</ETag><Size>9426368</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_amd64/converge</Key><LastModified>2016-10-24T16:45:49.000Z</LastModified><ETag>&quot;d8a711ba551b3d97728f1b0ebcc08c4e&quot;</ETag><Size>10633376</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_arm/converge</Key><LastModified>2016-10-24T16:45:49.000Z</LastModified><ETag>&quot;e4fb2f8bb2255e59701145c5e4badb83&quot;</ETag><Size>9597728</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_arm64/converge</Key><LastModified>2016-10-24T16:45:50.000Z</LastModified><ETag>&quot;dea3f0b09086ff94bf5efd94b92061e2&quot;</ETag><Size>10827744</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_ppc64/converge</Key><LastModified>2016-10-24T16:45:50.000Z</LastModified><ETag>&quot;37caded29f7a0bea2ee715e7a6c2c8a1&quot;</ETag><Size>11155488</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_ppc64le/converge</Key><LastModified>2016-10-24T16:45:51.000Z</LastModified><ETag>&quot;fdbf8dbf87c376b01faa6659e709f1fc&quot;</ETag><Size>11155456</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/converge_0.3.0-beta1-12-gb1f50f8-dirty_solaris_amd64/converge</Key><LastModified>2016-10-24T16:45:52.000Z</LastModified><ETag>&quot;9a2c959757c7c49afbfd66b7d5a4b0a0&quot;</ETag><Size>10499616</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_darwin_386.tar.gz</Key><LastModified>2016-10-24T16:45:52.000Z</LastModified><ETag>&quot;c252faecee60c0f5e389c9bbae6a1128&quot;</ETag><Size>3716032</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_darwin_amd64.tar.gz</Key><LastModified>2016-10-24T16:45:53.000Z</LastModified><ETag>&quot;bd68b6ef18e61f37ffbfa211b54adb09&quot;</ETag><Size>3899005</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_freebsd_386.tar.gz</Key><LastModified>2016-10-24T16:45:53.000Z</LastModified><ETag>&quot;67fd80e52445aabc40c8e674ac31f49c&quot;</ETag><Size>3521685</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_freebsd_amd64.tar.gz</Key><LastModified>2016-10-24T16:45:53.000Z</LastModified><ETag>&quot;d7352db53ac800b808a3fb36d701dd16&quot;</ETag><Size>3705037</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_freebsd_arm.tar.gz</Key><LastModified>2016-10-24T16:45:53.000Z</LastModified><ETag>&quot;d8115a31f74f7a47a2dc9a319b911e67&quot;</ETag><Size>3426766</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_386.tar.gz</Key><LastModified>2016-10-24T16:45:54.000Z</LastModified><ETag>&quot;679c335829668b1597f477c30b2a2d02&quot;</ETag><Size>3528651</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_amd64.tar.gz</Key><LastModified>2016-10-24T16:45:54.000Z</LastModified><ETag>&quot;874cc1759ffbdddae96770523facd2bd&quot;</ETag><Size>3734813</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_arm.tar.gz</Key><LastModified>2016-10-24T16:45:55.000Z</LastModified><ETag>&quot;0d32cb44cefd59da60d2d3031520167a&quot;</ETag><Size>3429441</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_arm64.tar.gz</Key><LastModified>2016-10-24T16:45:55.000Z</LastModified><ETag>&quot;5771db869c3fd29079a7586fe50be00d&quot;</ETag><Size>3453130</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_ppc64.tar.gz</Key><LastModified>2016-10-24T16:45:56.000Z</LastModified><ETag>&quot;3d412605ea0947767f8af017445b8f70&quot;</ETag><Size>3470075</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_linux_ppc64le.tar.gz</Key><LastModified>2016-10-24T16:45:56.000Z</LastModified><ETag>&quot;0d2476c7b4fedf617da2863b99d6640b&quot;</ETag><Size>3427674</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-12-gb1f50f8/tgz/converge_0.3.0-beta1-12-gb1f50f8-dirty_solaris_amd64.tar.gz</Key><LastModified>2016-10-24T16:45:56.000Z</LastModified><ETag>&quot;d951ec247501d0b2fe822db536bbb06a&quot;</ETag><Size>3678314</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_darwin_386/converge</Key><LastModified>2016-10-24T16:38:47.000Z</LastModified><ETag>&quot;451a8d47eff7bc5cbc3be685519eb226&quot;</ETag><Size>10492160</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_darwin_amd64/converge</Key><LastModified>2016-10-24T16:38:48.000Z</LastModified><ETag>&quot;50d66d8668b73d2b51d0b645dd781772&quot;</ETag><Size>11777472</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_freebsd_386/converge</Key><LastModified>2016-10-24T16:38:50.000Z</LastModified><ETag>&quot;d9f4c60154338724e548c69de3a0db49&quot;</ETag><Size>9352416</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_freebsd_amd64/converge</Key><LastModified>2016-10-24T16:38:50.000Z</LastModified><ETag>&quot;92e7e263f352232986ee3422ffcf5aae&quot;</ETag><Size>10566048</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_freebsd_arm/converge</Key><LastModified>2016-10-24T16:38:51.000Z</LastModified><ETag>&quot;2bb40915ae79be0678de826e3e01250d&quot;</ETag><Size>9601432</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_linux_386/converge</Key><LastModified>2016-10-24T16:38:51.000Z</LastModified><ETag>&quot;61370b80d87f87885ea8fe107854dfb7&quot;</ETag><Size>9426368</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_linux_amd64/converge</Key><LastModified>2016-10-24T16:38:52.000Z</LastModified><ETag>&quot;90ab37718ec92e03f52528b0779c67b4&quot;</ETag><Size>10633280</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_linux_arm/converge</Key><LastModified>2016-10-24T16:38:53.000Z</LastModified><ETag>&quot;ba3174c53672bef1d7c87170fe825115&quot;</ETag><Size>9597728</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_linux_arm64/converge</Key><LastModified>2016-10-24T16:38:53.000Z</LastModified><ETag>&quot;c0b8c8e44a6a7ebee2c8adb02d158112&quot;</ETag><Size>10827744</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_linux_ppc64/converge</Key><LastModified>2016-10-24T16:38:53.000Z</LastModified><ETag>&quot;0042e566fb8c46474c3d86ab64cca730&quot;</ETag><Size>11155488</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_linux_ppc64le/converge</Key><LastModified>2016-10-24T16:38:54.000Z</LastModified><ETag>&quot;010d075673f4c4e3dfe9963340a733c8&quot;</ETag><Size>11155456</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/converge_0.3.0-beta1-15-gda31a75_solaris_amd64/converge</Key><LastModified>2016-10-24T16:38:54.000Z</LastModified><ETag>&quot;8134a897f2df01d469e0deba8e219143&quot;</ETag><Size>10499616</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/tgz/converge_0.3.0-beta1-15-gda31a75_darwin_386.tar.gz</Key><LastModified>2016-10-24T16:38:55.000Z</LastModified><ETag>&quot;d39f7a35e45565a2d6ee814e2ecead65&quot;</ETag><Size>3715987</Size><StorageClass>STANDARD</StorageClass></Contents><Contents><Key>0.3.0-beta1-15-gda31a75/tgz/converge_0.3.0-beta1-15-gda31a75_darwin_amd64.tar.gz</Key><LastModified>2016-10-24T16:38:55.000Z</LastModified><ETag>&quot;7f182f8895b9e653c4d4ce510381ecde&quot;</ETag><Size>3898918</Size><StorageClass>STANDARD</StorageClass></Contents></ListBucketResult>
"""


decodedXml : Value
decodedXml =
    """
<person>
    <name>noah</name>
    <age max="100">50</age>
</person>
<person>
    <name>josh</name>
    <age max="100">57</age>
</person>
    """
        |> decode
        |> Result.toMaybe
        |> Maybe.withDefault null


type alias Person =
    { name : String
    , age : Int
    }


person : Value -> Result String Person
person value =
    Result.map2
        (\name age ->
            { name = name
            , age = age
            }
        )
        (tag "name" string value)
        (tag "age" int value)


people : List Person
people =
    tags "person" decodedXml
        |> collect person


correctPeople : List Person
correctPeople =
    [ { name = "noah", age = 50 }
    , { name = "josh", age = 57 }
    ]
